import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import comp202.fall2007.a5.comparator.ComparatorFactory;
import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.util.Song;


public class COMP202AlbumSelectionPanel extends JPanel implements
		MusicSelectionComponent {
	// *** CONSTANTS
	private static final int SPACING = 5;

	private static final int NUMBER_COMBO_BOXES = 1;
	private static final int ALBUMS= 0;

	private static final String[] COMBO_BOX_ACTION_COMMANDS = {
			"add-all-albums" };

	private static final String[] COMBO_BOX_BUTTON_LABELS = { "Add all" };

	private static final String[] ALBUM_SELECTION_COMBO_BOX_LABELS = {
			"Album: " };

	private static final String[] ALBUM_SELECTION_COMBO_BOX_INITIAL_VALUES = {
			"All albums"};

	private static final int NUMBER_SELECTION_BUTTONS = 2;
	private static final int SELECT_ALL = 0;
	private static final int ADD_SELECTED = 1;

	private static final String[] SELECTION_ACTION_COMMANDS = { "select-all",
			"add-selected" };

	private static final String[] SELECTION_BUTTON_LABELS = { "Select All",
			"Add Selected" };

	// *** INSTANCE VARIABLES

	// Application objects
	private MusicCollection collection;
	private FilterFactory factory;

	// GUI Components
	private JComboBox[] albumSelectionComboBoxes;
	private DefaultComboBoxModel[] albumSelectionComboBoxModels;
	private JButton[] comboBoxButtons;
	private JButton[] selectionButtons;
	private JList albumSelectionTrackList;
	private DefaultListModel albumSelectionTrackListModel;

	// Other components
	private PlaylistComponent playlist;

	// *** CONSTRUCTOR

	public COMP202AlbumSelectionPanel(MusicCollection collection,
			PlaylistComponent playlist) {
		JScrollPane listScroller;
		JPanel labelPanel;
		JPanel buttonPanel;
		JPanel comboBoxPanel;
		JPanel topPanel;
		JPanel middlePanel;
		JPanel bottomPanel;

		this.collection = collection;
		this.playlist = playlist;
		this.factory = new FilterFactory();

		this.albumSelectionComboBoxes = new JComboBox[NUMBER_COMBO_BOXES];
		this.albumSelectionComboBoxModels = new DefaultComboBoxModel[NUMBER_COMBO_BOXES];
		this.comboBoxButtons = new JButton[NUMBER_COMBO_BOXES];

		for (int i = 0; i < this.albumSelectionComboBoxes.length; i++) {
			this.comboBoxButtons[i] = new JButton(COMBO_BOX_BUTTON_LABELS[i]);
			this.comboBoxButtons[i]
					.setActionCommand(COMBO_BOX_ACTION_COMMANDS[i]);
			this.comboBoxButtons[i].setEnabled(i == ALBUMS);

			this.albumSelectionComboBoxModels[i] = new DefaultComboBoxModel();
			this.albumSelectionComboBoxModels[i]
					.addElement(ALBUM_SELECTION_COMBO_BOX_INITIAL_VALUES[i]);

			this.albumSelectionComboBoxes[i] = new JComboBox(
					this.albumSelectionComboBoxModels[i]);
			this.albumSelectionComboBoxes[i].setEnabled(i == ALBUMS);
			this.albumSelectionComboBoxes[i].setPreferredSize(new Dimension(
					215, 26));
		}

		this.albumSelectionTrackListModel = new DefaultListModel();
		this.albumSelectionTrackList = new JList(
				this.albumSelectionTrackListModel);
		this.albumSelectionTrackList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		this.selectionButtons = new JButton[NUMBER_SELECTION_BUTTONS];
		for (int i = 0; i < this.selectionButtons.length; i++) {
			this.selectionButtons[i] = new JButton(SELECTION_BUTTON_LABELS[i]);
			this.selectionButtons[i]
					.setActionCommand(SELECTION_ACTION_COMMANDS[i]);
			this.selectionButtons[i].setEnabled(false);
		}

		labelPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		comboBoxPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1,
				SPACING, SPACING));
		buttonPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		for (int i = 0; i < this.albumSelectionComboBoxes.length; i++) {
			labelPanel.add(new JLabel(ALBUM_SELECTION_COMBO_BOX_LABELS[i]));
			comboBoxPanel.add(this.albumSelectionComboBoxes[i]);
			buttonPanel.add(this.comboBoxButtons[i]);
		}

		topPanel = new JPanel(new BorderLayout(SPACING, 0));
		topPanel.add(labelPanel, BorderLayout.LINE_START);
		topPanel.add(comboBoxPanel, BorderLayout.CENTER);
		topPanel.add(buttonPanel, BorderLayout.LINE_END);

		listScroller = new JScrollPane(this.albumSelectionTrackList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		listScroller.setPreferredSize(new Dimension(300, 150));

		middlePanel = new JPanel(new BorderLayout(0, SPACING));
		middlePanel.add(new JLabel("Tracks: "), BorderLayout.PAGE_START);
		middlePanel.add(listScroller, BorderLayout.CENTER);

		bottomPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		for (int i = 0; i < this.selectionButtons.length; i++) {
			bottomPanel.add(this.selectionButtons[i]);
		}

		this.setLayout(new BorderLayout(0, 3 * SPACING));
		this.add(topPanel, BorderLayout.PAGE_START);
		this.add(middlePanel, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.PAGE_END);

		// Register event listeners
		this.albumSelectionComboBoxes[ALBUMS]
				.addItemListener(new AlbumsComboBoxListener());
		this.albumSelectionTrackList
				.addListSelectionListener(new TrackListSelectionListener());

		this.comboBoxButtons[ALBUMS]
				.addActionListener(new AddAllAlbumsButtonListener());

		this.selectionButtons[SELECT_ALL]
				.addActionListener(new SelectAllButtonListener());
		this.selectionButtons[ADD_SELECTED]
				.addActionListener(new AddSelectedButtonListener());
	}

	// *** LISTENERS THAT RESPOND TO BUTTON CLICKS

	private class AddAllAlbumsButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			Playlist songSet;
			String albumTitle;
			SongFilter filter;
			int selectedIndex;

			albumTitle = albumSelectionComboBoxes[ALBUMS].getSelectedItem()
					.toString();

			selectedIndex = albumSelectionComboBoxes[ALBUMS]
					.getSelectedIndex();
			if (selectedIndex == 0) {
				filter = factory.createEverythingFilter();
				songSet = collection.getSongs(filter);
				songSet
						.sort(ComparatorFactory.ASCENDING_ALBUM_DISC_TRACK_COMPARATOR);

			} else {
				albumTitle = albumSelectionComboBoxes[ALBUMS]
						.getSelectedItem().toString();
				songSet = collection.getSongs(albumTitle);
			}

			playlist.addAll(songSet);
		}
	}

	private class SelectAllButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			albumSelectionTrackList.setSelectionInterval(0,
					albumSelectionTrackListModel.getSize() - 1);
		}
	}

	private class AddSelectedButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int[] selected;

			selected = albumSelectionTrackList.getSelectedIndices();
			for (int i = 0; i < selected.length; i++) {
				playlist.add((Song) albumSelectionTrackListModel
						.get(selected[i]));
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO COMBO BOX SELECTION

	private class AlbumsComboBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent event) {
			Playlist songSet;
			String selectedAlbum;
			int selectedIndex;

			if (event.getStateChange() == ItemEvent.SELECTED) {
				selectedIndex = albumSelectionComboBoxes[ALBUMS]
						.getSelectedIndex();
				if (selectedIndex == 0) {
					albumSelectionTrackListModel.removeAllElements();
					selectionButtons[SELECT_ALL].setEnabled(false);
				} else {
					selectedAlbum = event.getItem().toString();
					songSet = collection
							.getSongs(selectedAlbum);

					albumSelectionTrackListModel.removeAllElements();
					for (int i = 0; i < songSet.getSize(); i++) {
						albumSelectionTrackListModel
								.addElement(songSet.get(i));
					}

					selectionButtons[SELECT_ALL].setEnabled(true);
				}
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO LIST SELECTION

	private class TrackListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent event) {
			int[] selected;

			selected = albumSelectionTrackList.getSelectedIndices();
			selectionButtons[ADD_SELECTED].setEnabled(selected.length != 0);
		}
	}

	// *** PRIVATE METHODS

	private void updateList(int list, ArrayList<String> newValues) {
		albumSelectionComboBoxModels[list].removeAllElements();

		albumSelectionComboBoxModels[list]
				.addElement(ALBUM_SELECTION_COMBO_BOX_INITIAL_VALUES[list]);
		for (String value : newValues) {
			albumSelectionComboBoxModels[list].addElement(value);
		}
	}

	// *** METHODS REQUIRED BY THE INTERFACE

	public void updateState() {
		this.updateList(ALBUMS, this.collection.getAllAlbums());
	}
}
