import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import comp202.fall2007.a5.comparator.ComparatorFactory;
import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.filter.StringMatchType;
import comp202.fall2007.a5.util.Song;

public class COMP202ArtistSelectionPanel extends JPanel implements
		MusicSelectionComponent {
	// *** CONSTANTS
	private static final int SPACING = 5;

	private static final int NUMBER_COMBO_BOXES = 2;
	private static final int ARTISTS = 0;
	private static final int CONTRIBUTED_ALBUMS = 1;

	private static final String[] COMBO_BOX_ACTION_COMMANDS = {
			"add-all-artists", "add-all-contributed-albums" };

	private static final String[] COMBO_BOX_BUTTON_LABELS = { "Add all",
			"Add all" };

	private static final String[] ARTIST_SELECTION_COMBO_BOX_LABELS = {
			"Artist: ", "Album: " };

	private static final String[] ARTIST_SELECTION_COMBO_BOX_INITIAL_VALUES = {
			"All artists", "All contributed albums" };

	private static final int NUMBER_SELECTION_BUTTONS = 2;
	private static final int SELECT_ALL = 0;
	private static final int ADD_SELECTED = 1;

	private static final String[] SELECTION_ACTION_COMMANDS = { "select-all",
			"add-selected" };

	private static final String[] SELECTION_BUTTON_LABELS = { "Select All",
			"Add Selected" };

	// *** INSTANCE VARIABLES

	// Application objects
	private MusicCollection collection;
	private FilterFactory factory;

	// GUI Components
	private JComboBox[] artistSelectionComboBoxes;
	private DefaultComboBoxModel[] artistSelectionComboBoxModels;
	private JButton[] comboBoxButtons;
	private JButton[] selectionButtons;
	private JList artistSelectionTrackList;
	private DefaultListModel artistSelectionTrackListModel;

	// Other components
	private PlaylistComponent playlist;

	// *** CONSTRUCTOR

	public COMP202ArtistSelectionPanel(MusicCollection collection,
			PlaylistComponent playlist) {
		JScrollPane listScroller;
		JPanel labelPanel;
		JPanel buttonPanel;
		JPanel comboBoxPanel;
		JPanel topPanel;
		JPanel middlePanel;
		JPanel bottomPanel;

		this.collection = collection;
		this.playlist = playlist;
		this.factory = new FilterFactory();

		this.artistSelectionComboBoxes = new JComboBox[NUMBER_COMBO_BOXES];
		this.artistSelectionComboBoxModels = new DefaultComboBoxModel[NUMBER_COMBO_BOXES];
		this.comboBoxButtons = new JButton[NUMBER_COMBO_BOXES];

		for (int i = 0; i < this.artistSelectionComboBoxes.length; i++) {
			this.comboBoxButtons[i] = new JButton(COMBO_BOX_BUTTON_LABELS[i]);
			this.comboBoxButtons[i]
					.setActionCommand(COMBO_BOX_ACTION_COMMANDS[i]);
			this.comboBoxButtons[i].setEnabled(i == ARTISTS);

			this.artistSelectionComboBoxModels[i] = new DefaultComboBoxModel();
			this.artistSelectionComboBoxModels[i]
					.addElement(ARTIST_SELECTION_COMBO_BOX_INITIAL_VALUES[i]);

			this.artistSelectionComboBoxes[i] = new JComboBox(
					this.artistSelectionComboBoxModels[i]);
			this.artistSelectionComboBoxes[i].setEnabled(i == ARTISTS);
			this.artistSelectionComboBoxes[i].setPreferredSize(new Dimension(
					215, 26));
		}

		this.artistSelectionTrackListModel = new DefaultListModel();
		this.artistSelectionTrackList = new JList(
				this.artistSelectionTrackListModel);
		this.artistSelectionTrackList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		this.selectionButtons = new JButton[NUMBER_SELECTION_BUTTONS];
		for (int i = 0; i < this.selectionButtons.length; i++) {
			this.selectionButtons[i] = new JButton(SELECTION_BUTTON_LABELS[i]);
			this.selectionButtons[i]
					.setActionCommand(SELECTION_ACTION_COMMANDS[i]);
			this.selectionButtons[i].setEnabled(false);
		}

		labelPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		comboBoxPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1,
				SPACING, SPACING));
		buttonPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		for (int i = 0; i < this.artistSelectionComboBoxes.length; i++) {
			labelPanel.add(new JLabel(ARTIST_SELECTION_COMBO_BOX_LABELS[i]));
			comboBoxPanel.add(this.artistSelectionComboBoxes[i]);
			buttonPanel.add(this.comboBoxButtons[i]);
		}

		topPanel = new JPanel(new BorderLayout(SPACING, 0));
		topPanel.add(labelPanel, BorderLayout.LINE_START);
		topPanel.add(comboBoxPanel, BorderLayout.CENTER);
		topPanel.add(buttonPanel, BorderLayout.LINE_END);

		listScroller = new JScrollPane(this.artistSelectionTrackList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		listScroller.setPreferredSize(new Dimension(300, 150));

		middlePanel = new JPanel(new BorderLayout(0, SPACING));
		middlePanel.add(new JLabel("Tracks: "), BorderLayout.PAGE_START);
		middlePanel.add(listScroller, BorderLayout.CENTER);

		bottomPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		for (int i = 0; i < this.selectionButtons.length; i++) {
			bottomPanel.add(this.selectionButtons[i]);
		}

		this.setLayout(new BorderLayout(0, 3 * SPACING));
		this.add(topPanel, BorderLayout.PAGE_START);
		this.add(middlePanel, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.PAGE_END);

		// Register event listeners
		this.artistSelectionComboBoxes[ARTISTS]
				.addItemListener(new ArtistsComboBoxListener());
		this.artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
				.addItemListener(new ContributedAlbumsComboBoxListener());
		this.artistSelectionTrackList
				.addListSelectionListener(new TrackListSelectionListener());

		this.comboBoxButtons[ARTISTS]
				.addActionListener(new AddAllArtistsButtonListener());
		this.comboBoxButtons[CONTRIBUTED_ALBUMS]
				.addActionListener(new AddContributedAlbumsButtonListener());

		this.selectionButtons[SELECT_ALL]
				.addActionListener(new SelectAllButtonListener());
		this.selectionButtons[ADD_SELECTED]
				.addActionListener(new AddSelectedButtonListener());
	}

	// *** LISTENERS THAT RESPOND TO BUTTON CLICKS

	private class AddAllArtistsButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			Playlist songSet;
			SongFilter filter;
			String artistName;
			int selectedIndex;

			selectedIndex = artistSelectionComboBoxes[ARTISTS]
					.getSelectedIndex();
			if (selectedIndex == 0) {
				filter = factory.createEverythingFilter();
			} else {
				artistName = artistSelectionComboBoxes[ARTISTS]
						.getSelectedItem().toString();
				filter = factory.createArtistFilter(artistName,
						StringMatchType.EQUALS);
			}

			songSet = collection.getSongs(filter);
			songSet
					.sort(ComparatorFactory.ASCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR);

			playlist.addAll(songSet);
		}
	}

	private class AddContributedAlbumsButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			Playlist songSet;
			String artistName;
			String albumTitle;
			SongFilter filter;
			int selectedIndex;

			artistName = artistSelectionComboBoxes[ARTISTS].getSelectedItem()
					.toString();

			selectedIndex = artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
					.getSelectedIndex();
			if (selectedIndex == 0) {
				filter = factory.createArtistFilter(artistName,
						StringMatchType.EQUALS);
				songSet = collection.getSongs(filter);
				songSet
						.sort(ComparatorFactory.ASCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR);

			} else {
				albumTitle = artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
						.getSelectedItem().toString();
				songSet = collection.getSongs(artistName, albumTitle);
			}

			playlist.addAll(songSet);
		}
	}

	private class SelectAllButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			artistSelectionTrackList.setSelectionInterval(0,
					artistSelectionTrackListModel.getSize() - 1);
		}
	}

	private class AddSelectedButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int[] selected;

			selected = artistSelectionTrackList.getSelectedIndices();
			for (int i = 0; i < selected.length; i++) {
				playlist.add((Song) artistSelectionTrackListModel
						.get(selected[i]));
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO COMBO BOX SELECTION

	private class ArtistsComboBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent event) {
			String selectedItem;
			int selectedIndex;

			if (event.getStateChange() == ItemEvent.SELECTED) {
				selectedIndex = artistSelectionComboBoxes[ARTISTS]
						.getSelectedIndex();
				if (selectedIndex == 0) {
					artistSelectionTrackListModel.removeAllElements();

					artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
							.setSelectedIndex(0);
					artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
							.setEnabled(false);

					comboBoxButtons[CONTRIBUTED_ALBUMS].setEnabled(false);

				} else {
					selectedItem = event.getItem().toString();
					comboBoxButtons[CONTRIBUTED_ALBUMS].setEnabled(true);
					artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
							.setEnabled(true);
					updateList(CONTRIBUTED_ALBUMS, collection
							.getContributedAlbums(selectedItem));
				}
			}
		}
	}

	private class ContributedAlbumsComboBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent event) {
			Playlist songSet;
			String selectedArtist;
			String selectedAlbum;
			int selectedIndex;

			if (event.getStateChange() == ItemEvent.SELECTED) {
				selectedIndex = artistSelectionComboBoxes[CONTRIBUTED_ALBUMS]
						.getSelectedIndex();
				if (selectedIndex == 0) {
					artistSelectionTrackListModel.removeAllElements();
					selectionButtons[SELECT_ALL].setEnabled(false);
				} else {
					selectedArtist = artistSelectionComboBoxes[ARTISTS]
							.getSelectedItem().toString();
					selectedAlbum = event.getItem().toString();
					songSet = collection
							.getSongs(selectedArtist, selectedAlbum);

					artistSelectionTrackListModel.removeAllElements();
					for (int i = 0; i < songSet.getSize(); i++) {
						artistSelectionTrackListModel
								.addElement(songSet.get(i));
					}

					selectionButtons[SELECT_ALL].setEnabled(true);
				}
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO LIST SELECTION

	private class TrackListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent event) {
			int[] selected;

			selected = artistSelectionTrackList.getSelectedIndices();
			selectionButtons[ADD_SELECTED].setEnabled(selected.length != 0);
		}
	}

	// *** PRIVATE METHODS

	private void updateList(int list, ArrayList<String> newValues) {
		artistSelectionComboBoxModels[list].removeAllElements();

		artistSelectionComboBoxModels[list]
				.addElement(ARTIST_SELECTION_COMBO_BOX_INITIAL_VALUES[list]);
		for (String value : newValues) {
			artistSelectionComboBoxModels[list].addElement(value);
		}
	}

	// *** METHODS REQUIRED BY THE INTERFACE

	public void updateState() {
		this.updateList(ARTISTS, this.collection.getAllArtists());
	}
}
