import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.UIManager;

import comp202.fall2007.a5.loader.SongLoader;
import comp202.fall2007.a5.loader.SongLoaderListener;
import comp202.fall2007.a5.util.Song;

public class COMP202DirectorySelectionPanel extends JPanel {
	// *** CONSTANTS ***

	// Spacing constant
	private static final int SPACING = 5;

	// Directory selection buttons
	private static final int NUMBER_DIRECTORY_SELECTION_BUTTONS = 1;
	private static final int ADD_DIRECTORY = 0;

	// Directory selection action commands
	private static final String[] DIRECTORY_SELECTION_ACTION_COMMANDS = { "add-directory" };

	// Directory selection button labels
	private static final String[] DIRECTORY_SELECTION_BUTTON_LABELS = { "Add directory..." };

	// Directory selection labels
	private static int NUMBER_DIRECTORY_SELECTION_LABELS = 2;
	private static int SONG_COUNT = 0;
	private static int STATUS = 1;

	// Directory selection labels text
	private static final String[] DIRECTORY_SELECTION_LABELS_TEXT = {
			"Total songs found: ", "Loading status: " };

	// Directory selection label values
	private static final String IN_PROGRESS_STATUS_LABEL = "In progress...";
	private static final String STOPPED_STATUS_LABEL = "Stopped";

	// *** INSTANCE VARIABLES ***

	// Application
	private MusicCollection collection;
	private MusicCollectionBuilder collectionBuilder;
	private SongLoader songLoader;

	// GUI Components
	private JButton[] addDirectoryButtons;
	private JLabel[] addDirectoryLabels;
	private JList directoryList;
	private DefaultListModel directoryListModel;
	private JFileChooser directoryFileChooser;
	
	// Other components
	private ArrayList<MusicSelectionComponent> musicSelectionListeners;

	// *** CONSTRUCTOR

	public COMP202DirectorySelectionPanel(MusicCollection collection) {
		JScrollPane listScroller;
		JPanel labelPanel;

		// Initialize the collection
		this.collection = collection;
		this.collectionBuilder = new MusicCollectionBuilder();
		this.songLoader = new SongLoader();
		this.songLoader.addListener(this.collectionBuilder);
		
		this.musicSelectionListeners = new ArrayList<MusicSelectionComponent>();

		// Set up GUI
		this.directoryFileChooser = new JFileChooser();
		this.directoryFileChooser
				.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

		this.directoryListModel = new DefaultListModel();
		this.directoryList = new JList(this.directoryListModel);
		this.directoryList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listScroller = new JScrollPane(this.directoryList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		listScroller.setPreferredSize(new Dimension(300, 135));

		this.addDirectoryButtons = new JButton[NUMBER_DIRECTORY_SELECTION_BUTTONS];
		this.addDirectoryButtons[ADD_DIRECTORY] = new JButton(
				DIRECTORY_SELECTION_BUTTON_LABELS[ADD_DIRECTORY]);
		this.addDirectoryButtons[ADD_DIRECTORY]
				.setActionCommand(DIRECTORY_SELECTION_ACTION_COMMANDS[ADD_DIRECTORY]);
		this.addDirectoryButtons[ADD_DIRECTORY].setEnabled(true);
		this.addDirectoryButtons[ADD_DIRECTORY]
				.addActionListener(new AddDirectoryButtonListener());

		this.addDirectoryLabels = new JLabel[NUMBER_DIRECTORY_SELECTION_LABELS];
		labelPanel = new JPanel(new GridLayout(this.addDirectoryLabels.length,
				2, SPACING, SPACING));
		for (int i = 0; i < this.addDirectoryLabels.length; i++) {
			if (i == SONG_COUNT) {
				this.addDirectoryLabels[i] = new JLabel("0");
			} else {
				this.addDirectoryLabels[i] = new JLabel(STOPPED_STATUS_LABEL);
			}
			labelPanel.add(new JLabel(DIRECTORY_SELECTION_LABELS_TEXT[i]));
			labelPanel.add(this.addDirectoryLabels[i]);
		}

		this.setLayout(new BorderLayout(0, 3 * SPACING));
		this.add(this.addDirectoryButtons[ADD_DIRECTORY],
				BorderLayout.PAGE_START);
		this.add(listScroller, BorderLayout.CENTER);
		this.add(labelPanel, BorderLayout.PAGE_END);
	}

	// *** BUTTON EVENT LISTENERS

	private class AddDirectoryButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			Thread loaderThread;
			File directory;
			int result;

			result = directoryFileChooser
					.showOpenDialog(COMP202DirectorySelectionPanel.this);
			if (result == JFileChooser.APPROVE_OPTION) {
				directory = directoryFileChooser.getSelectedFile();
				directoryListModel.addElement(directory.getAbsolutePath());

				addDirectoryLabels[STATUS].setText(IN_PROGRESS_STATUS_LABEL);
				addDirectoryButtons[ADD_DIRECTORY].setEnabled(false);
				loaderThread = new Thread(new CollectionLoaderRunnable(
						directory));
				loaderThread.start();
			}
		}
	}

	// *** THREAD EVENT LISTENERS

	// SongLoader loaded a Song
	private class LoadedSongEventListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			addDirectoryLabels[SONG_COUNT].setText(Integer
					.toString(collectionBuilder.getSongCount()));
		}
	}

	// SongLoader is done loading
	private class DoneLoadingEventListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			addDirectoryLabels[STATUS].setText(STOPPED_STATUS_LABEL);
			addDirectoryButtons[ADD_DIRECTORY].setEnabled(true);
			
			for (MusicSelectionComponent component : musicSelectionListeners) {
				component.updateState();
			}
		}
	}

	// *** SUPPORT CLASSES

	// Custom SongLoaderListener
	private class MusicCollectionBuilder extends SongLoaderListener {
		private int songCount;
		private Timer loadedSongTimer;
		private Timer doneLoadingTimer;

		public MusicCollectionBuilder() {
			this.songCount = 0;
			
			this.loadedSongTimer = new Timer(0, new LoadedSongEventListener());
			this.loadedSongTimer.setRepeats(false);
			
			this.doneLoadingTimer = new Timer(0, new DoneLoadingEventListener());
			this.doneLoadingTimer.setRepeats(false);
		}

		public synchronized int getSongCount() {
			return this.songCount;
		}

		private synchronized void incrementSongCount() {
			this.songCount++;
		}

		public void finishedLoading() {
			this.doneLoadingTimer.start();
		}

		public void loadedSong(Song song) {
			if (collection.getSong(song.getFilePath()) == null) {
				collection.add(song);
				this.incrementSongCount();
				this.loadedSongTimer.start();
			}
		}

		public void loadingInterrupted() {
		}

		public void startedLoading() {
		}
	}

	// Custom Runnable
	private class CollectionLoaderRunnable implements Runnable {
		private File directory;

		private CollectionLoaderRunnable(File directory) {
			this.directory = directory;
		}

		public void run() {
			songLoader.load(this.directory);
		}
	}
	
	// *** OTHER PUBLIC METHODS
	
	public void addMusicSelectionComponent(MusicSelectionComponent component) {
		this.musicSelectionListeners.add(component);
	}
	
	// Test
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception exception) {
			InternalError error = new InternalError();
			error.initCause(exception);
			throw error;
		}

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Container contentPane;
				JFrame frame;
				JTabbedPane collectionPanes;
				MusicCollection collection;
				COMP202DirectorySelectionPanel directorySelectionPanel;
				COMP202ArtistSelectionPanel  artistSelectionPanel;
				COMP202FilterCreationPanel filterCreationPanel;
				COMP202FilterSelectionPanel filterSelectionPanel;
				
				collection = new ThreadSafeMusicCollection();
				directorySelectionPanel = new COMP202DirectorySelectionPanel(
						collection);
				artistSelectionPanel = new COMP202ArtistSelectionPanel(collection, null);
				filterSelectionPanel = new COMP202FilterSelectionPanel(collection, null);
				filterCreationPanel = new COMP202FilterCreationPanel(filterSelectionPanel);
				
				directorySelectionPanel.addMusicSelectionComponent(artistSelectionPanel);

				collectionPanes = new JTabbedPane();
				collectionPanes.add("Directories", directorySelectionPanel);
				collectionPanes.add("Artists", artistSelectionPanel);
				collectionPanes.add("Filter Creation", filterCreationPanel);
				
				frame = new JFrame("Test Frame");
				contentPane = frame.getContentPane();
				contentPane.setLayout(new BorderLayout(SPACING, SPACING));
				contentPane.add(collectionPanes, BorderLayout.CENTER);

				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.pack();
				frame.setVisible(true);

			}
		});
	}
}
