import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.filter.NumberMatchType;
import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.filter.StringMatchType;

public class COMP202FilterCreationPanel extends JPanel {
	// *** CONSTANTS

	// Spacing
	private static final int SPACING = 5;

	// Combo boxes
	private static final int NUMBER_COMBO_BOXES = 2;
	private static final int ATTRIBUTE = 0;
	private static final int MATCH_TYPE = 1;

	// Buttons
	private static final int NUMBER_BUTTONS = 7;
	private static final int ADD = 0;
	private static final int REMOVE = 1;
	private static final int CLEAR = 2;
	private static final int AND = 3;
	private static final int OR = 4;
	private static final int NOT = 5;
	private static final int CREATE = 6;

	// Action commands
	private static final String[] FILTER_CREATION_ACTION_COMMANDS = { "add",
			"remove", "clear", "and", "or", "not", "create" };

	// Button labels
	private static final String[] FILTER_CREATION_BUTTON_LABELS = { "Add",
			"Remove", "Clear", "AND", "OR", "NOT", "Create" };

	// Button initial states
	private static final boolean[] FILTER_CREATION_BUTTON_INITIAL_STATES = {
			true, false, true, false, false, false, false };

	// Match type enum
	private static enum MatchType {
		NUMBER, STRING, PATH
	}

	// Match attributes
	private static final int TITLE = 0;
	private static final int ARTIST = 1;
	private static final int ALBUM = 2;
	private static final int PATH = 3;
	private static final int LENGTH = 4;
	private static final int DISC = 5;
	private static final int TRACK = 6;
	
	private static final String[] MATCH_ATTRIBUTES = { "Title", "Artist",
			"Album", "Path", "Length", "Disc", "Track" };

	// Match types for every attribute
	private static final MatchType[] MATCH_TYPES = { MatchType.STRING,
			MatchType.STRING, MatchType.STRING, MatchType.PATH,
			MatchType.NUMBER, MatchType.NUMBER, MatchType.NUMBER, };

	// All possible number match types
	private static final NumberMatchType[] NUMBER_MATCH_TYPES = {
			NumberMatchType.EQUAL, NumberMatchType.AT_LEAST,
			NumberMatchType.AT_MOST };

	// All possible String match types
	private static final StringMatchType[] STRING_MATCH_TYPES = {
			StringMatchType.EQUALS, StringMatchType.CONTAINS };

	// All possible path match types
	private static final String[] PATH_MATCH_TYPES = { "equals" };

	// Check box label
	private static final String CHECK_BOX_LABEL = "Match any of the criteria";
	
	// Error messages
	private static final String NON_NUMERIC_ARGUMENT = "Values for this type of filter must be numeric.";

	// *** INSTANCE VARIABLES

	// Underlying filters list
	private FilterFactory factory;
	private int currentlySelectedAttribute;

	// GUI Components
	private JComboBox[] filterCreationComboBoxes;
	private DefaultComboBoxModel[] filterCreationComboBoxModels;
	private JButton[] filterCreationButtons;
	private JTextField filterCreationTextField;
	private JList filterList;
	private DefaultListModel filterListModel;
	private JCheckBox filterCreationCheckBox;
	
	private FilterSelectionComponent filterSelection;

	// *** CONSTRUCTOR

	public COMP202FilterCreationPanel(FilterSelectionComponent filterSelection) {
		JScrollPane listScroller;
		JPanel comboBoxPanel;
		JPanel buttonPanel;
		JPanel topPanel;
		JPanel bottomPanel;

		this.factory = new FilterFactory();
		this.currentlySelectedAttribute = 0;
		
		this.filterSelection = filterSelection;

		this.filterCreationComboBoxModels = new DefaultComboBoxModel[NUMBER_COMBO_BOXES];
		this.filterCreationComboBoxes = new JComboBox[NUMBER_COMBO_BOXES];
		for (int i = 0; i < filterCreationComboBoxes.length; i++) {
			this.filterCreationComboBoxModels[i] = new DefaultComboBoxModel();
			this.filterCreationComboBoxes[i] = new JComboBox(
					filterCreationComboBoxModels[i]);
		}
		this.filterCreationComboBoxes[MATCH_TYPE].setPreferredSize(new Dimension(72, 24));

		for (String attribute : MATCH_ATTRIBUTES) {
			this.filterCreationComboBoxModels[ATTRIBUTE].addElement(attribute);
		}

		for (StringMatchType matchType : STRING_MATCH_TYPES) {
			this.filterCreationComboBoxModels[MATCH_TYPE].addElement(matchType);
		}

		this.filterCreationTextField = new JTextField(20);

		this.filterCreationButtons = new JButton[NUMBER_BUTTONS];
		for (int i = 0; i < this.filterCreationButtons.length; i++) {
			this.filterCreationButtons[i] = new JButton(
					FILTER_CREATION_BUTTON_LABELS[i]);
			this.filterCreationButtons[i]
					.setActionCommand(FILTER_CREATION_ACTION_COMMANDS[i]);
			this.filterCreationButtons[i]
					.setEnabled(FILTER_CREATION_BUTTON_INITIAL_STATES[i]);
		}

		this.filterListModel = new DefaultListModel();
		this.filterList = new JList(this.filterListModel);

		this.filterCreationCheckBox = new JCheckBox(CHECK_BOX_LABEL);

		comboBoxPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		for (int i = 0; i < this.filterCreationComboBoxes.length; i++) {
			comboBoxPanel.add(this.filterCreationComboBoxes[i]);
		}
		comboBoxPanel.add(this.filterCreationTextField);

		buttonPanel = new JPanel(new GridLayout(2, 3, SPACING, SPACING));
		for (int i = 0; i < 6; i++) {
			buttonPanel.add(this.filterCreationButtons[i]);
		}

		topPanel = new JPanel(new BorderLayout());
		topPanel.add(comboBoxPanel, BorderLayout.PAGE_START);
		topPanel.add(buttonPanel, BorderLayout.CENTER);

		listScroller = new JScrollPane(this.filterList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		listScroller.setPreferredSize(new Dimension(300, 150));

		bottomPanel = new JPanel(new GridLayout(1, 2, SPACING, SPACING));
		bottomPanel.add(this.filterCreationCheckBox);
		bottomPanel.add(this.filterCreationButtons[CREATE]);

		this.setLayout(new BorderLayout(0, 3 * SPACING));
		this.add(topPanel, BorderLayout.PAGE_START);
		this.add(listScroller, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.PAGE_END);

		// Add listeners
		this.filterCreationButtons[ADD]
				.addActionListener(new AddButtonListener());
		this.filterCreationButtons[REMOVE]
				.addActionListener(new RemoveButtonListener());
		this.filterCreationButtons[CLEAR]
				.addActionListener(new ClearButtonListener());
		this.filterCreationButtons[AND]
				.addActionListener(new AndButtonListener());
		this.filterCreationButtons[OR]
				.addActionListener(new OrButtonListener());
		this.filterCreationButtons[NOT]
				.addActionListener(new NotButtonListener());
		this.filterCreationButtons[CREATE]
				.addActionListener(new CreateButtonListener());

		this.filterCreationComboBoxes[ATTRIBUTE]
				.addItemListener(new AttributeComboBoxListener());

		this.filterList.addListSelectionListener(new FilterListListener());
	}

	// *** BUTTON LISTENERS

	private class AddButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			SongFilter filter;
			StringMatchType stringMatchType;
			NumberMatchType numberMatchType;
			int selectedAttribute;
			int value;
			
			selectedAttribute = filterCreationComboBoxes[ATTRIBUTE].getSelectedIndex();
			filter = null;
			if (MATCH_TYPES[selectedAttribute] == MatchType.STRING) {
				stringMatchType = (StringMatchType)filterCreationComboBoxes[MATCH_TYPE].getSelectedItem();
				switch (selectedAttribute) {
				case TITLE:
					filter = factory.createTitleFilter(filterCreationTextField.getText(), stringMatchType);
					break;
				case ARTIST:
					filter = factory.createArtistFilter(filterCreationTextField.getText(), stringMatchType);
					break;
				case ALBUM:
					filter = factory.createAlbumFilter(filterCreationTextField.getText(), stringMatchType);
					break;
				}
			}
			if (MATCH_TYPES[selectedAttribute] == MatchType.PATH) {
				switch(selectedAttribute) {
				case PATH:
					filter = factory.createPathFilter(new File(filterCreationTextField.getText()));
					break;
				}
			}
			if (MATCH_TYPES[selectedAttribute] == MatchType.NUMBER) {
				try {
					value = Integer.parseInt(filterCreationTextField.getText());
					numberMatchType = (NumberMatchType)filterCreationComboBoxes[MATCH_TYPE].getSelectedItem();
					switch (selectedAttribute) {
					case LENGTH:
						filter = factory.createLengthFilter(value, numberMatchType);
						break;
					case DISC:
						filter = factory.createDiscFilter(value, numberMatchType);
						break;
					case TRACK:
						filter = factory.createTrackFilter(value, numberMatchType);
						break;
					}
				} catch (NumberFormatException exception) {
					JOptionPane.showMessageDialog(COMP202FilterCreationPanel.this, NON_NUMERIC_ARGUMENT);
				}
			}
			
			if (filter != null) {
				filterListModel.addElement(filter);
				filterCreationButtons[CREATE].setEnabled(true);
			}
		}
	}

	private class RemoveButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int[] selected;

			selected = filterList.getSelectedIndices();
			for (int i = selected.length - 1; i >= 0; i--) {
				filterListModel.removeElementAt(selected[i]);
			}

			filterCreationButtons[CREATE].setEnabled(!filterListModel.isEmpty());
		}
	}

	private class ClearButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			filterListModel.removeAllElements();

			filterCreationButtons[CREATE].setEnabled(false);
		}
	}

	private abstract class MultiOperatorButtonListener implements
			ActionListener {
		protected abstract SongFilter createFilter(
				ArrayList<SongFilter> subFilters);

		public void actionPerformed(ActionEvent event) {
			ArrayList<SongFilter> subFilters;
			SongFilter filter;
			SongFilter newFilter;
			int[] selected;

			selected = filterList.getSelectedIndices();
			subFilters = new ArrayList<SongFilter>();
			for (int i = selected.length - 1; i >= 0; i--) {
				filter = (SongFilter)filterListModel.get(selected[i]);
				subFilters.add(0, filter);

				if (i != 0) {
					filterListModel.remove(selected[i]);
				} else {
					newFilter = this.createFilter(subFilters);
					filterListModel.set(selected[i], newFilter);
				}
			}
		}
	}

	private class AndButtonListener extends MultiOperatorButtonListener {
		protected SongFilter createFilter(ArrayList<SongFilter> subFilters) {
			return factory.createAndFilter(subFilters);
		}
	}

	private class OrButtonListener extends MultiOperatorButtonListener {
		protected SongFilter createFilter(ArrayList<SongFilter> subFilters) {
			return factory.createOrFilter(subFilters);
		}
	}

	private class NotButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int index;
			SongFilter oldFilter;
			SongFilter newFilter;

			index = filterList.getSelectedIndex();
			oldFilter = (SongFilter)filterListModel.get(index);
			newFilter = factory.createNotFilter(oldFilter);
			filterListModel.set(index, newFilter);
		}
	}

	private class CreateButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			SongFilter[] filters;
			SongFilter filter;
			
			filters = new SongFilter[filterListModel.getSize()];
			for (int i = 0; i < filters.length; i++) {
				filters[i] = (SongFilter)filterListModel.getElementAt(i);
			}
			
			if (filterCreationCheckBox.isSelected()) {
				filter = factory.createOrFilter(filters);
			} else {
				filter = factory.createAndFilter(filters);
			}
			filterListModel.removeAllElements();
			filterSelection.addFilter(filter);
		}
	}

	// *** COMBO BOX LISTENERS

	private class AttributeComboBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent event) {
			int selectedIndex;

			if (event.getStateChange() == ItemEvent.SELECTED) {
				selectedIndex = filterCreationComboBoxes[ATTRIBUTE]
						.getSelectedIndex();
				if (MATCH_TYPES[currentlySelectedAttribute] != MATCH_TYPES[selectedIndex]) {
					filterCreationComboBoxModels[MATCH_TYPE]
							.removeAllElements();

					if (MATCH_TYPES[selectedIndex] == MatchType.STRING) {
						for (StringMatchType matchType : STRING_MATCH_TYPES) {
							filterCreationComboBoxModels[MATCH_TYPE]
									.addElement(matchType);
						}
					}

					if (MATCH_TYPES[selectedIndex] == MatchType.NUMBER) {
						for (NumberMatchType matchType : NUMBER_MATCH_TYPES) {
							filterCreationComboBoxModels[MATCH_TYPE]
									.addElement(matchType);
						}
					}

					if (MATCH_TYPES[selectedIndex] == MatchType.PATH) {
						for (String matchType : PATH_MATCH_TYPES) {
							filterCreationComboBoxModels[MATCH_TYPE]
									.addElement(matchType);
						}
					}

					filterCreationComboBoxes[MATCH_TYPE]
							.setEnabled(MATCH_TYPES[selectedIndex] != MatchType.PATH);
					currentlySelectedAttribute = selectedIndex;
				}
			}
		}
	}

	private class FilterListListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent event) {
			int selected[];

			selected = filterList.getSelectedIndices();
			filterCreationButtons[REMOVE].setEnabled(selected.length > 0);
			filterCreationButtons[AND].setEnabled(selected.length > 1);
			filterCreationButtons[OR].setEnabled(selected.length > 1);
			filterCreationButtons[NOT].setEnabled(selected.length == 1);
		}
	}
}
