import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.util.Song;

public class COMP202FilterSelectionPanel extends JPanel implements
		FilterSelectionComponent, MusicSelectionComponent {

	// *** CONSTANTS
	private static final int SPACING = 5;

	private static final int NUMBER_COMBO_BOXES = 1;
	private static final int FILTERS = 0;

	private static final String[] COMBO_BOX_ACTION_COMMANDS = { "remove" };

	private static final String[] COMBO_BOX_BUTTON_LABELS = { "Remove" };

	private static final String[] ALBUM_SELECTION_COMBO_BOX_LABELS = { "Filter: " };

	private static final String[] ALBUM_SELECTION_COMBO_BOX_INITIAL_VALUES = { "All filters" };

	private static final int NUMBER_SELECTION_BUTTONS = 2;
	private static final int SELECT_ALL = 0;
	private static final int ADD_SELECTED = 1;

	private static final String[] SELECTION_ACTION_COMMANDS = { "select-all",
			"add-selected" };

	private static final String[] SELECTION_BUTTON_LABELS = { "Select All",
			"Add Selected" };

	// *** INSTANCE VARIABLES

	// Application objects
	private MusicCollection collection;

	// GUI Components
	private JComboBox[] filterSelectionComboBoxes;
	private DefaultComboBoxModel[] filterSelectionComboBoxModels;
	private JButton[] comboBoxButtons;
	private JButton[] selectionButtons;
	private JList filterTrackList;
	private DefaultListModel filterTrackListModel;

	// Other components
	private PlaylistComponent playlist;

	// *** CONSTRUCTOR

	public COMP202FilterSelectionPanel(MusicCollection collection,
			PlaylistComponent playlist) {
		JScrollPane listScroller;
		JPanel labelPanel;
		JPanel buttonPanel;
		JPanel comboBoxPanel;
		JPanel topPanel;
		JPanel middlePanel;
		JPanel bottomPanel;

		this.collection = collection;
		this.playlist = playlist;

		this.filterSelectionComboBoxes = new JComboBox[NUMBER_COMBO_BOXES];
		this.filterSelectionComboBoxModels = new DefaultComboBoxModel[NUMBER_COMBO_BOXES];
		this.comboBoxButtons = new JButton[NUMBER_COMBO_BOXES];

		for (int i = 0; i < this.filterSelectionComboBoxes.length; i++) {
			this.comboBoxButtons[i] = new JButton(COMBO_BOX_BUTTON_LABELS[i]);
			this.comboBoxButtons[i]
					.setActionCommand(COMBO_BOX_ACTION_COMMANDS[i]);
			this.comboBoxButtons[i].setEnabled(false);

			this.filterSelectionComboBoxModels[i] = new DefaultComboBoxModel();
			this.filterSelectionComboBoxModels[i]
					.addElement(ALBUM_SELECTION_COMBO_BOX_INITIAL_VALUES[i]);

			this.filterSelectionComboBoxes[i] = new JComboBox(
					this.filterSelectionComboBoxModels[i]);
			this.filterSelectionComboBoxes[i].setEnabled(i == FILTERS);
			this.filterSelectionComboBoxes[i].setPreferredSize(new Dimension(
			291, 26));
		}


		this.filterTrackListModel = new DefaultListModel();
		this.filterTrackList = new JList(this.filterTrackListModel);
		this.filterTrackList
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		this.selectionButtons = new JButton[NUMBER_SELECTION_BUTTONS];
		for (int i = 0; i < this.selectionButtons.length; i++) {
			this.selectionButtons[i] = new JButton(SELECTION_BUTTON_LABELS[i]);
			this.selectionButtons[i]
					.setActionCommand(SELECTION_ACTION_COMMANDS[i]);
			this.selectionButtons[i].setEnabled(false);
		}

		labelPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		comboBoxPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1,
				SPACING, SPACING));
		buttonPanel = new JPanel(new GridLayout(NUMBER_COMBO_BOXES, 1, SPACING,
				SPACING));
		for (int i = 0; i < this.filterSelectionComboBoxes.length; i++) {
			labelPanel.add(new JLabel(ALBUM_SELECTION_COMBO_BOX_LABELS[i]));
			comboBoxPanel.add(this.filterSelectionComboBoxes[i]);
			buttonPanel.add(this.comboBoxButtons[i]);
		}

		topPanel = new JPanel(new BorderLayout(SPACING, 0));
		topPanel.add(labelPanel, BorderLayout.LINE_START);
		topPanel.add(comboBoxPanel, BorderLayout.CENTER);
		topPanel.add(buttonPanel, BorderLayout.LINE_END);

		listScroller = new JScrollPane(this.filterTrackList,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		listScroller.setPreferredSize(new Dimension(300, 150));

		middlePanel = new JPanel(new BorderLayout(0, SPACING));
		middlePanel.add(new JLabel("Tracks: "), BorderLayout.PAGE_START);
		middlePanel.add(listScroller, BorderLayout.CENTER);

		bottomPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		for (int i = 0; i < this.selectionButtons.length; i++) {
			bottomPanel.add(this.selectionButtons[i]);
		}

		this.setLayout(new BorderLayout(0, 3 * SPACING));
		this.add(topPanel, BorderLayout.PAGE_START);
		this.add(middlePanel, BorderLayout.CENTER);
		this.add(bottomPanel, BorderLayout.PAGE_END);

		// Register event listeners
		this.filterSelectionComboBoxes[FILTERS]
				.addItemListener(new AlbumsComboBoxListener());
		this.filterTrackList
				.addListSelectionListener(new TrackListSelectionListener());

		this.comboBoxButtons[FILTERS]
				.addActionListener(new RemoveFilterButtonListener());

		this.selectionButtons[SELECT_ALL]
				.addActionListener(new SelectAllButtonListener());
		this.selectionButtons[ADD_SELECTED]
				.addActionListener(new AddSelectedButtonListener());
	}

	// *** LISTENERS THAT RESPOND TO BUTTON CLICKS

	private class RemoveFilterButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int selectedIndex;

			selectedIndex = filterSelectionComboBoxes[FILTERS]
					.getSelectedIndex();

			if (selectedIndex != 0) {
				filterSelectionComboBoxModels[FILTERS]
						.removeElementAt(selectedIndex);
				filterTrackListModel.removeAllElements();
				filterSelectionComboBoxes[FILTERS].setSelectedIndex(0);
			}
		}
	}

	private class SelectAllButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			filterTrackList.setSelectionInterval(0, filterTrackListModel
					.getSize() - 1);
		}
	}

	private class AddSelectedButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int[] selected;

			selected = filterTrackList.getSelectedIndices();
			for (int i = 0; i < selected.length; i++) {
				playlist.add((Song) filterTrackListModel.get(selected[i]));
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO COMBO BOX SELECTION

	private class AlbumsComboBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent event) {
			Playlist songSet;
			SongFilter filter;
			int selectedIndex;

			if (event.getStateChange() == ItemEvent.SELECTED) {
				selectedIndex = filterSelectionComboBoxes[FILTERS]
						.getSelectedIndex();
				if (selectedIndex == 0) {
					filterTrackListModel.removeAllElements();
					comboBoxButtons[FILTERS].setEnabled(false);

					selectionButtons[SELECT_ALL].setEnabled(false);
				} else {
					comboBoxButtons[FILTERS].setEnabled(true);

					filter = (SongFilter) filterSelectionComboBoxes[FILTERS]
							.getSelectedItem();
					songSet = collection.getSongs(filter);

					filterTrackListModel.removeAllElements();
					for (int i = 0; i < songSet.getSize(); i++) {
						filterTrackListModel.addElement(songSet.get(i));
					}

					selectionButtons[SELECT_ALL].setEnabled(true);
				}
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO LIST SELECTION

	private class TrackListSelectionListener implements ListSelectionListener {
		public void valueChanged(ListSelectionEvent event) {
			int[] selected;

			selected = filterTrackList.getSelectedIndices();
			selectionButtons[ADD_SELECTED].setEnabled(selected.length != 0);
		}
	}

	// *** METHODS REQUIRED BY THE INTERFACE

	public void updateState() {
		this.filterSelectionComboBoxes[FILTERS].setSelectedIndex(0);
	}

	public void addFilter(SongFilter filter) {
		this.filterSelectionComboBoxModels[FILTERS].addElement(filter);
	}
}
