import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import javazoom.jlgui.basicplayer.BasicPlayer;

public class COMP202MusicPlayerInterface {
	private static final int SPACING = 5;

	private static final String FRAME_TITLE = "COMP-202A Music Player";
	private static final String DIRECTORY_PANE_TITLE = "Directories";
	private static final String ARTISTS_PANE_TITLE = "Artists";
	private static final String ALBUMS_PANE_TITLE = "Albums";
	private static final String FILTER_SELECTION_TITLE = "Filter Selection";
	private static final String FILTER_CREATION_TITLE = "Filter Creation";

	private JFrame frame;

	public COMP202MusicPlayerInterface() {
		BasicPlayer player;
		MusicCollection collection;
		COMP202TrackInfoPanel trackInfoPanel;
		COMP202PlayStatePanel playStatePanel;
		COMP202PlayAttributesPanel playAttributesPanel;
		COMP202PlaylistSortPanel playlistSortPanel;
		COMP202PlaylistPanel playlistPanel;
		COMP202PlaylistControlPanel playlistControlPanel;
		COMP202DirectorySelectionPanel directorySelectionPanel;
		COMP202ArtistSelectionPanel artistSelectionPanel;
		COMP202AlbumSelectionPanel albumSelectionPanel;
		COMP202FilterSelectionPanel filterSelectionPanel;
		COMP202FilterCreationPanel filterCreationPanel;
		JTabbedPane collectionPanes;
		JPanel topLeftPanel;
		JPanel rightPanel;
		JPanel leftPanel;
		Container contentPane;

		player = new BasicPlayer();
		collection = new ThreadSafeMusicCollection();

		// Create the components
		playlistPanel = new COMP202PlaylistPanel();
		playlistSortPanel = new COMP202PlaylistSortPanel(playlistPanel);
		playlistControlPanel = new COMP202PlaylistControlPanel(playlistPanel,
				collection);

		trackInfoPanel = new COMP202TrackInfoPanel();
		playStatePanel = new COMP202PlayStatePanel(player, playlistPanel);
		playAttributesPanel = new COMP202PlayAttributesPanel(player,
				playStatePanel);

		directorySelectionPanel = new COMP202DirectorySelectionPanel(collection);
		artistSelectionPanel = new COMP202ArtistSelectionPanel(collection,
				playlistPanel);
		albumSelectionPanel = new COMP202AlbumSelectionPanel(collection,
				playlistPanel);
		filterSelectionPanel = new COMP202FilterSelectionPanel(collection, playlistPanel);
		filterCreationPanel = new COMP202FilterCreationPanel(filterSelectionPanel);

		// Connect them
		playStatePanel.setTrackInfo(trackInfoPanel);
		playStatePanel.setPlayAttributesControl(playAttributesPanel);
		playAttributesPanel.setPlaylist(playlistPanel);
		playlistPanel.setPlayStateControl(playStatePanel);
		playlistPanel.setPlaylistControl(playlistControlPanel);

		directorySelectionPanel
				.addMusicSelectionComponent(artistSelectionPanel);
		directorySelectionPanel.addMusicSelectionComponent(albumSelectionPanel);

		// Place them on the frame
		collectionPanes = new JTabbedPane();
		collectionPanes.add(DIRECTORY_PANE_TITLE, directorySelectionPanel);
		collectionPanes.add(ARTISTS_PANE_TITLE, artistSelectionPanel);
		collectionPanes.add(ALBUMS_PANE_TITLE, albumSelectionPanel);
		collectionPanes.add(FILTER_SELECTION_TITLE, filterSelectionPanel);
		collectionPanes.add(FILTER_CREATION_TITLE, filterCreationPanel);

		topLeftPanel = new JPanel();
		topLeftPanel
				.setLayout(new BoxLayout(topLeftPanel, BoxLayout.PAGE_AXIS));
		topLeftPanel.add(trackInfoPanel);
		topLeftPanel.add(Box.createRigidArea(new Dimension(0, 3 * SPACING)));
		topLeftPanel.add(playStatePanel);
		topLeftPanel.add(Box.createRigidArea(new Dimension(0, 3 * SPACING)));
		topLeftPanel.add(playAttributesPanel);

		leftPanel = new JPanel(new BorderLayout(0, 3 * SPACING));
		leftPanel.add(topLeftPanel, BorderLayout.PAGE_START);
		leftPanel.add(collectionPanes, BorderLayout.CENTER);

		rightPanel = new JPanel(new BorderLayout(SPACING, SPACING));
		rightPanel.add(playlistSortPanel, BorderLayout.PAGE_START);
		rightPanel.add(playlistPanel, BorderLayout.CENTER);
		rightPanel.add(playlistControlPanel, BorderLayout.PAGE_END);

		this.frame = new JFrame(FRAME_TITLE);
		contentPane = this.frame.getContentPane();
		contentPane.setLayout(new BorderLayout(SPACING, SPACING));
		contentPane.add(leftPanel, BorderLayout.LINE_START);
		contentPane.add(rightPanel, BorderLayout.CENTER);

		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.frame.pack();
	}

	public void startInterface() {
		this.frame.setVisible(true);
	}

	public static void start() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception exception) {
			InternalError error = new InternalError();
			error.initCause(exception);
			throw error;
		}

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				COMP202MusicPlayerInterface player;

				player = new COMP202MusicPlayerInterface();
				player.startInterface();
			}
		});
	}
}
