import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import javazoom.jlgui.basicplayer.BasicController;
import javazoom.jlgui.basicplayer.BasicPlayer;
import javazoom.jlgui.basicplayer.BasicPlayerEvent;
import javazoom.jlgui.basicplayer.BasicPlayerException;
import javazoom.jlgui.basicplayer.BasicPlayerListener;

import comp202.fall2007.a5.util.Song;

public class COMP202PlayAttributesPanel extends JPanel implements
		PlayAttributesComponent {
	// *** CONSTANTS

	// Spacing constant
	private static final int SPACING = 5;

	// Play control sliders
	private static final int NUMBER_PLAY_CONTROL_SLIDERS = 3;
	private static final int TIME = 0;
	private static final int VOLUME = 1;
	private static final int BALANCE = 2;

	// Play control slider attributes
	private static final int MIN = 0;
	private static final int MAX = 1;
	private static final int INITIAL = 2;
	private static final int MAJOR_TICK_SPACING = 3;
	private static final int MINOR_TICK_SPACING = 4;

	// Play control slider attribute values
	private static final int[][] SLIDER_VALUES = { { 0, 10000, 0, 0, 0 },
			{ 0, 100, 50, 25, 1 }, { -100, 100, 0, 100, 1 }, };

	private static final int TIME_SLIDER_SIZE = SLIDER_VALUES[TIME][MAX]
			- SLIDER_VALUES[TIME][MIN];

	// Play control slider labels
	private static final String[] CONTROL_SLIDERS_TEXT = { "Time: ",
			"Volume: ", "Balance: " };

	// Play control slider value labels
	private static final String[] CONTROL_SLIDER_LABELS_INITIAL_VALUES = {
			"0:00", Integer.toString(SLIDER_VALUES[VOLUME][INITIAL]),
			Integer.toString(SLIDER_VALUES[VOLUME][INITIAL]), };

	// Balance slider labels
	private static final String BALANCE_LEFT_LABEL = "Left";
	private static final String BALANCE_CENTER_LABEL = "Center";
	private static final String BALANCE_RIGHT_LABEL = "Right";

	// Error messages
	private static final String CHANGE_VOLUME_ERROR = "Could not change volume.";
	private static final String CHANGE_BALANCE_ERROR = "Could not change balance.";
	private static final String SEEK_ERROR = "Could not seek.";

	// *** INSTANCE VARIABLES

	private BasicPlayer player;
	private SeekState seekState;
	private DecimalFormat timeFormatter;

	// Play control sliders
	private JSlider[] playControlSliders;

	// Play control information labels
	private JLabel[] playControlLabels;

	// Other components
	private PlayStateComponent playStateControl;
	private PlaylistComponent playlist;

	// *** CONSTRUCTOR

	public COMP202PlayAttributesPanel(BasicPlayer player,
			PlayStateComponent playStateControl) {
		HashMap<Integer, ActionListener> listeners;
		ProgressSliderListener progressListener;
		ChangeListener listener;
		BasicPlayerListener playerListener;
		Hashtable balanceLabels;
		JPanel labelsPanel;
		JPanel slidersPanel;

		// Initialize the player and its seek state
		this.player = player;
		this.seekState = SeekState.NORMAL;

		// Initialize the formatter for time units
		this.timeFormatter = new DecimalFormat("00");

		// Initialize the other components
		this.playStateControl = playStateControl;
		this.playlist = null;

		// Create and setup variable labels and sliders
		this.playControlLabels = new JLabel[NUMBER_PLAY_CONTROL_SLIDERS];
		this.playControlSliders = new JSlider[NUMBER_PLAY_CONTROL_SLIDERS];
		labelsPanel = new JPanel(new GridLayout(NUMBER_PLAY_CONTROL_SLIDERS, 2));
		slidersPanel = new JPanel(
				new GridLayout(NUMBER_PLAY_CONTROL_SLIDERS, 1));

		progressListener = null;
		for (int i = 0; i < NUMBER_PLAY_CONTROL_SLIDERS; i++) {
			// Labels
			this.playControlLabels[i] = new JLabel(
					CONTROL_SLIDER_LABELS_INITIAL_VALUES[i],
					SwingConstants.RIGHT);
			labelsPanel.add(new JLabel(CONTROL_SLIDERS_TEXT[i]));
			labelsPanel.add(this.playControlLabels[i]);

			// Sliders
			this.playControlSliders[i] = new JSlider(JSlider.HORIZONTAL,
					SLIDER_VALUES[i][MIN], SLIDER_VALUES[i][MAX],
					SLIDER_VALUES[i][INITIAL]);
			this.playControlSliders[i].setPaintLabels(true);

			if (i == TIME) {
				this.playControlSliders[i].setEnabled(false);
				progressListener = new ProgressSliderListener();
				listener = progressListener;
			} else {
				this.playControlSliders[i].setEnabled(true);

				this.playControlSliders[i]
						.setMajorTickSpacing(SLIDER_VALUES[i][MAJOR_TICK_SPACING]);
				this.playControlSliders[i]
						.setMinorTickSpacing(SLIDER_VALUES[i][MINOR_TICK_SPACING]);
				if (i == BALANCE) {
					listener = new BalanceSliderListener(
							this.playControlLabels[i], SLIDER_VALUES[i][MAX],
							CHANGE_BALANCE_ERROR);

					balanceLabels = new Hashtable();
					balanceLabels.put(SLIDER_VALUES[i][MIN], new JLabel(
							BALANCE_LEFT_LABEL));
					balanceLabels.put(SLIDER_VALUES[i][INITIAL], new JLabel(
							BALANCE_CENTER_LABEL));
					balanceLabels.put(SLIDER_VALUES[i][MAX], new JLabel(
							BALANCE_RIGHT_LABEL));
					this.playControlSliders[i].setLabelTable(balanceLabels);
				} else {
					listener = new VolumeSliderListener(
							this.playControlLabels[i], SLIDER_VALUES[i][MAX],
							CHANGE_VOLUME_ERROR);
				}
			}
			this.playControlSliders[i].addChangeListener(listener);
			slidersPanel.add(this.playControlSliders[i]);
		}

		this.setLayout(new BorderLayout(SPACING, SPACING));
		this.add(labelsPanel, BorderLayout.LINE_START);
		this.add(slidersPanel, BorderLayout.CENTER);

		// Set up the player event listeners
		listeners = new HashMap<Integer, ActionListener>();
		listeners.put(BasicPlayerEvent.SEEKED, new SeekedEventListener());
		playerListener = new TimerPlayerListener(listeners);
		this.player.addBasicPlayerListener(playerListener);
		this.player.addBasicPlayerListener(new SeekPlayerListener(
				progressListener));
	}

	// *** LISTENERS THAT RESPOND TO SLIDER EVENTS

	// Base class
	private abstract class SliderListener implements ChangeListener {
		private JLabel label;
		private int denominator;
		private String errorMessage;

		public SliderListener(JLabel label, int denominator, String errorMessage) {
			this.label = label;
			this.denominator = denominator;
			this.errorMessage = errorMessage;
		}

		protected abstract void setValue(double newValue)
				throws BasicPlayerException;

		public void stateChanged(ChangeEvent event) {
			JSlider source;
			int value;

			source = (JSlider) event.getSource();
			value = source.getValue();

			if (playStateControl.getPlayerState() == PlayStateComponent.PlayerState.PLAY) {
				try {
					setValue(value / (double) this.denominator);
					this.label.setText(Integer.toString(value));
				} catch (BasicPlayerException exception) {
					JOptionPane.showMessageDialog(
							COMP202PlayAttributesPanel.this, this.errorMessage);
				}
			} else {
				this.label.setText(Integer.toString(value));
			}
		}
	}

	// Volume slider
	private class VolumeSliderListener extends SliderListener {
		public VolumeSliderListener(JLabel label, int denominator,
				String errorMessage) {
			super(label, denominator, errorMessage);
		}

		protected void setValue(double newValue) throws BasicPlayerException {
			player.setGain(newValue);
		}
	}

	// Balance slider
	private class BalanceSliderListener extends SliderListener {
		public BalanceSliderListener(JLabel label, int denominator,
				String errorMessage) {
			super(label, denominator, errorMessage);
		}

		protected void setValue(double newValue) throws BasicPlayerException {
			player.setPan(newValue);
		}
	}

	// Progress slider; also responds to progress events fired from the player
	// thread
	private class ProgressSliderListener implements ActionListener,
			ChangeListener {
		// These variables are in seconds
		private int timeElapsed;
		private int duration;
		private int bytesLength;

		public ProgressSliderListener() {
			this.timeElapsed = 0;
			this.duration = 0;
			this.bytesLength = 0;
		}

		public synchronized int getBytesLength() {
			return this.bytesLength;
		}

		public synchronized void setBytesLength(int bytesLength) {
			this.bytesLength = bytesLength;
		}

		public synchronized int getTimeElapsed() {
			return this.timeElapsed;
		}

		public synchronized void setTimeElapsed(int timeElapsed) {
			this.timeElapsed = timeElapsed;
		}

		public synchronized int getDuration() {
			return this.duration;
		}

		public synchronized void setDuration(int duration) {
			this.duration = duration;
		}

		public void actionPerformed(ActionEvent event) {
			int totalTime;
			int position;
			int currentTime;

			// Do not request the progress slider to change its position while
			// the user is moving it.
			if (getSeekState() == SeekState.NORMAL) {
				currentTime = this.getTimeElapsed();
				totalTime = this.getDuration();

				position = currentTime * TIME_SLIDER_SIZE / totalTime;
				playControlSliders[TIME].setValue(position);
			}
		}

		public void stateChanged(ChangeEvent event) {
			JSlider source;
			long skipBytes;
			double ratio;
			int position;

			source = (JSlider) event.getSource();
			position = source.getValue();

			if (source.getValueIsAdjusting()) {
				// User started moving the progress slider. Set the flag.
				setSeekState(SeekState.MOVING);

				// Set the time in the label to correspond to progress bar
				// position
				ratio = (double) position / TIME_SLIDER_SIZE;
				playControlLabels[TIME]
						.setText(formatTime((int) (getDuration() * ratio)));

			} else {
				if (getSeekState() == SeekState.MOVING) {
					// User was moving the progress slider, but stopped. Seek.
					skipBytes = getBytesLength() * (long) position
							/ TIME_SLIDER_SIZE;
					try {
						player.seek(skipBytes);
						ratio = (double) position / TIME_SLIDER_SIZE;
						playControlLabels[TIME]
								.setText(formatTime((int) (getDuration() * ratio)));
						// Set the time in the label to correspond to progress
						// bar
						// position
						setSeekState(SeekState.SEEKING);
					} catch (BasicPlayerException exception) {
						setSeekState(SeekState.NORMAL);
						JOptionPane.showMessageDialog(
								COMP202PlayAttributesPanel.this, SEEK_ERROR);
					}
				} else {
					// Set the time in the label to correspond to time elapsed
					// as computed by the progress() method
					playControlLabels[TIME]
							.setText(formatTime(getTimeElapsed()));
				}
				// Otherwise, this event was generated by setting the slider
				// value as a result of events fired by the player thread.
				// Ignore.
			}
		}
	}

	// *** LISTENERS THAT RESPOND TO EVENTS FIRED FROM THE PLAYER THREAD

	// Seeked event
	private class SeekedEventListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			setSeekState(SeekState.NORMAL);
		}
	}

	// *** BASIC PLAYER LISTENER

	// Player listener that listens that sets up the progress slider and updates
	// it as the player progresses.
	// The methods it defines will not be invoked from the event-dispatching
	// thread, so they must not modify any GUI components. Instead, they should
	// fire events back into the event dispatching thread.
	private class SeekPlayerListener implements BasicPlayerListener {
		private ProgressSliderListener progressListener;
		private Timer progressTimer;

		public SeekPlayerListener(ProgressSliderListener progressListener) {
			this.progressListener = progressListener;

			this.progressTimer = new Timer(0, progressListener);
			this.progressTimer.setRepeats(false);
			this.progressTimer.setCoalesce(false);
		}

		public void opened(Object stream, Map properties) {
			final String durationKey = "duration";
			final String lengthKey = "audio.length.bytes";
			Integer bytesLength;
			Long duration;
			Song track;

			if (properties.containsKey(durationKey)) {
				duration = (Long) properties.get(durationKey);
				this.progressListener
						.setDuration((int) (duration.longValue() / 1000000));
			} else {
				if ((playlist != null) && !playlist.isEmpty()) {
					track = playlist.get(playlist.getCurrentPosition());
					this.progressListener.setDuration(track.getLength());
				} else {
					this.progressListener.setDuration(1);
				}
			}

			if (properties.containsKey(lengthKey)) {
				bytesLength = (Integer) properties.get(lengthKey);
				this.progressListener.setBytesLength(bytesLength.intValue());
			}
		}

		public void progress(int bytesRead, long microseconds, byte[] pcmdata,
				Map properties) {
			int time;
			double progress;

			if (getSeekState() != SeekState.SEEKING) {
				progress = (double) bytesRead
						/ this.progressListener.getBytesLength();
				time = (int) (this.progressListener.getDuration() * progress);
				this.progressListener.setTimeElapsed(time);
				this.progressTimer.start();
			}
		}

		public void setController(BasicController controller) {
		}

		public void stateUpdated(BasicPlayerEvent event) {
		}
	}

	// *** PRIVATE HELPER METHODS

	private String formatTime(int totalSeconds) {
		int hours;
		int minutes;
		int seconds;

		hours = totalSeconds / 3600;
		minutes = (totalSeconds % 3600) / 60;
		seconds = totalSeconds % 60;

		return ((hours == 0) ? "" : this.timeFormatter.format(hours) + ":")
				+ this.timeFormatter.format(minutes) + ":"
				+ this.timeFormatter.format(seconds);
	}

	private synchronized void setSeekState(SeekState seekState) {
		this.seekState = seekState;
	}

	// *** PUBLIC METHOD REQUIRED BY THE INTERFACE

	public synchronized SeekState getSeekState() {
		return this.seekState;
	}

	public boolean isVolumeDisplayEnabled() {
		return this.playControlSliders[VOLUME].isEnabled();
	}

	public int getMinimumVolume() {
		return SLIDER_VALUES[VOLUME][MIN];
	}

	public int getMaximumVolume() {
		return SLIDER_VALUES[VOLUME][MAX];
	}

	public int getCurrentVolume() {
		return this.playControlSliders[VOLUME].getValue();
	}

	public boolean isBalanceDisplayEnabled() {
		return this.playControlSliders[BALANCE].isEnabled();
	}

	public void setBalanceDisplayEnabled(boolean enabled) {
		this.playControlSliders[BALANCE].setEnabled(enabled);
	}

	public int getMinimumAbsoluteBalance() {
		return SLIDER_VALUES[BALANCE][INITIAL];
	}

	public int getMaximumAbsoluteBalance() {
		return SLIDER_VALUES[BALANCE][MAX];
	}

	public int getCurrentBalance() {
		return this.playControlSliders[BALANCE].getValue();
	}

	public boolean isTimeDisplayEnabled() {
		return this.playControlSliders[TIME].isEnabled();
	}

	public void updateState() {
		if (this.playStateControl.getPlayerState() == PlayStateComponent.PlayerState.STOP) {
			this.playControlSliders[TIME].setEnabled(false);
			this.playControlSliders[TIME].setValue(0);
			this.playControlLabels[TIME].setText(formatTime(0));
		} else {
			this.playControlSliders[TIME].setEnabled(true);
		}
	}

	// *** OTHER PUBLIC METHODS

	public PlayStateComponent getPlayStateControl() {
		return playStateControl;
	}

	public void setPlayStateControl(PlayStateComponent playStateControl) {
		this.playStateControl = playStateControl;
	}

	public PlaylistComponent getPlaylist() {
		return playlist;
	}

	public void setPlaylist(PlaylistComponent playlist) {
		this.playlist = playlist;
	}
}