import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import comp202.fall2007.a5.util.file.ExtensionFileFilter;

public class COMP202PlaylistControlPanel extends JPanel implements
		PlaylistControlComponent {
	// *** CONSTANTS

	// File extension
	private static final String PLAYLIST_FILE_EXTENSION = ".202pls";

	// Spacing constant
	private static final int SPACING = 5;

	// Playlist control buttons
	private static final int NUMBER_PLAYLIST_CONTROL_BUTTONS = 6;
	private static final int MOVE_UP = 0;
	private static final int MOVE_DOWN = 1;
	private static final int DELETE = 2;
	private static final int CLEAR = 3;
	private static final int APPEND = 4;
	private static final int SAVE = 5;

	// Playlist control button action commands
	private static final String[] PLAYLIST_CONTROL_ACTION_COMMANDS = {
			"move_up", "move_down", "delete", "clear", "load", "save" };

	// Playlist control button labels
	private static final String[] PLAYLIST_CONTROL_BUTTON_LABELS = { "Move Up",
			"Move Down", "Delete", "Clear", "Append...", "Save..." };

	// Playlist control button initial states
	private static final boolean[] PLAYLIST_CONTROL_BUTTONS_INITIAL_STATES = {
			false, false, false, true, true, true };

	// Playlist control label
	private static final String NUMBER_POSITIONS_LABEL_TEXT = "# of positions: ";

	// Error messages
	private static final String NON_NUMERIC_MOVE_VALUE = "The value for the number of positions the song is to be moved must be numeric.";
	private static final String NON_POSITIVE_MOVE_VALUE = "The value for the number of positions the song is to be moved must be positive.";
	private static final String MOVE_UP_OUT_OF_BOUNDS = "A track cannot be moved up above the beginning of the playlist.";
	private static final String MOVE_DOWN_OUT_OF_BOUNDS = "A track cannot be moved down below the end of the playlist.";
	private static final String COULD_NOT_READ_PLAYLIST = "Could not append the contents of playlist file ";
	private static final String COULD_NOT_WRITE_PLAYLIST = "Could not write the current playlist to ";
	private static final String COULD_NOT_FIND_SONG = "The following files could not be loaded: \n";

	// *** INSTANCE VARIABLES

	private MusicCollection collection;

	private int[] selectedRows;

	// Swing Components
	private JButton[] playlistControlButtons;
	private JTextField numberPositionsTextField;
	private JFileChooser playlistFileChooser;

	// Other custom components
	private PlaylistComponent playlist;

	// *** CONSTRUCTOR

	public COMP202PlaylistControlPanel(PlaylistComponent playlist,
			MusicCollection collection) {
		this.collection = collection;
		this.selectedRows = null;
		this.playlist = playlist;

		this.playlistFileChooser = new JFileChooser();
		this.playlistFileChooser.setFileFilter(new PlaylistFilter());

		this.numberPositionsTextField = new JTextField("1");
		this.numberPositionsTextField.setHorizontalAlignment(JTextField.RIGHT);

		this.playlistControlButtons = new JButton[NUMBER_PLAYLIST_CONTROL_BUTTONS];
		for (int i = 0; i < this.playlistControlButtons.length; i++) {
			this.playlistControlButtons[i] = new JButton(
					PLAYLIST_CONTROL_BUTTON_LABELS[i]);
			this.playlistControlButtons[i]
					.setActionCommand(PLAYLIST_CONTROL_ACTION_COMMANDS[i]);
			this.playlistControlButtons[i]
					.setEnabled(PLAYLIST_CONTROL_BUTTONS_INITIAL_STATES[i]);
		}

		this.setLayout(new GridLayout(2, 4, SPACING, SPACING));
		this.add(new JLabel(NUMBER_POSITIONS_LABEL_TEXT));
		this.add(this.numberPositionsTextField);
		this.add(this.playlistControlButtons[MOVE_UP]);
		this.add(this.playlistControlButtons[MOVE_DOWN]);
		this.add(this.playlistControlButtons[DELETE]);
		this.add(this.playlistControlButtons[CLEAR]);
		this.add(this.playlistControlButtons[APPEND]);
		this.add(this.playlistControlButtons[SAVE]);

		this.playlistControlButtons[MOVE_UP]
				.addActionListener(new MoveUpButtonListener());
		this.playlistControlButtons[MOVE_DOWN]
				.addActionListener(new MoveDownButtonListener());
		this.playlistControlButtons[DELETE]
				.addActionListener(new DeleteButtonListener());
		this.playlistControlButtons[CLEAR]
				.addActionListener(new ClearButtonListener());
		this.playlistControlButtons[APPEND]
				.addActionListener(new AppendButtonListener());
		this.playlistControlButtons[SAVE]
				.addActionListener(new SaveButtonListener());

	}

	// Move-Up Button
	private class MoveUpButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int numberPositions;

			try {
				numberPositions = Integer.parseInt(numberPositionsTextField
						.getText());
				playlist.moveUp(selectedRows[0], numberPositions);
			} catch (NumberFormatException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						NON_NUMERIC_MOVE_VALUE);
			} catch (IndexOutOfBoundsException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						MOVE_UP_OUT_OF_BOUNDS);
			} catch (IllegalArgumentException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						NON_POSITIVE_MOVE_VALUE);
			}

		}
	}

	// Move-Down Button
	private class MoveDownButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			int numberPositions;

			try {
				numberPositions = Integer.parseInt(numberPositionsTextField
						.getText());

				playlist.moveDown(selectedRows[0], numberPositions);
			} catch (NumberFormatException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						NON_NUMERIC_MOVE_VALUE);
			} catch (IndexOutOfBoundsException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						MOVE_DOWN_OUT_OF_BOUNDS);
			} catch (IllegalArgumentException exception) {
				JOptionPane.showMessageDialog(COMP202PlaylistControlPanel.this,
						NON_POSITIVE_MOVE_VALUE);
			}

		}
	}

	// Delete button
	private class DeleteButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			for (int i = selectedRows.length - 1; i >= 0; i--) {
				playlist.removeSong(selectedRows[i]);
			}
		}
	}

	// Clear button
	private class ClearButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			playlist.clear();
		}
	}

	// Append button
	private class AppendButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			ArrayList<String> badFiles;
			Playlist newList;
			String badFilesString;
			File inputFile;
			int numberBadFiles;
			int result;

			result = playlistFileChooser
					.showOpenDialog(COMP202PlaylistControlPanel.this);
			if (result == JFileChooser.APPROVE_OPTION) {
				inputFile = playlistFileChooser.getSelectedFile();
				newList = new Playlist();

				try {
					badFiles = newList.load(inputFile, collection);
					if (!newList.isEmpty()) {
						playlist.addAll(newList);
					}

					numberBadFiles = badFiles.size();
					if (numberBadFiles != 0) {
						badFilesString = "";
						for (int i = 0; i < numberBadFiles; i++) {
							badFilesString = badFilesString + "- "
									+ badFiles.get(i);
							if (i != (numberBadFiles - 1)) {
								badFilesString = badFilesString + "\n";
							}
						}
						JOptionPane.showMessageDialog(
								COMP202PlaylistControlPanel.this,
								COULD_NOT_FIND_SONG + badFilesString);
					}
				} catch (IOException exception) {
					JOptionPane.showMessageDialog(
							COMP202PlaylistControlPanel.this,
							COULD_NOT_READ_PLAYLIST + inputFile.getName());
				}
			}
		}
	}

	// Save button
	private class SaveButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			File outputFile;
			int result;

			result = playlistFileChooser
					.showSaveDialog(COMP202PlaylistControlPanel.this);
			if (result == JFileChooser.APPROVE_OPTION) {
				outputFile = playlistFileChooser.getSelectedFile();
				try {
					playlist.save(outputFile);
				} catch (IOException exception) {
					JOptionPane.showMessageDialog(
							COMP202PlaylistControlPanel.this,
							COULD_NOT_WRITE_PLAYLIST + outputFile.getName());
				}
			}
		}
	}

	private class PlaylistFilter extends FileFilter {
		private ExtensionFileFilter workerFilter;

		public PlaylistFilter() {
			this.workerFilter = new ExtensionFileFilter(PLAYLIST_FILE_EXTENSION);
		}

		public boolean accept(File file) {
			return this.workerFilter.accept(file);
		}

		public String getDescription() {
			return "COMP-202A Playlist Files (*" + PLAYLIST_FILE_EXTENSION
					+ ")";
		}
	}

	// *** METHODS REQUIRED BY THE INTERFACE

	public void playlistUpdated() {
	}

	public void selectionUpdated(int[] selectedRows) {
		this.selectedRows = selectedRows;

		this.playlistControlButtons[DELETE]
				.setEnabled(this.selectedRows.length > 0);
		this.playlistControlButtons[MOVE_UP]
				.setEnabled(this.selectedRows.length == 1);
		this.playlistControlButtons[MOVE_DOWN]
				.setEnabled(this.selectedRows.length == 1);
	}

	// *** OTHER PUBLIC METHODS

	public PlaylistComponent getPlaylist() {
		return playlist;
	}

	public void setPlaylist(PlaylistComponent playlist) {
		this.playlist = playlist;
	}
}
