import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import comp202.fall2007.a5.comparator.ComparatorFactory;
import comp202.fall2007.a5.comparator.SongComparator;

public class COMP202PlaylistSortPanel extends JPanel {
	// *** CONSTANTS
	
	// Spacing constant
	private static final int SPACING = 5;

	// Sort button constants
	private static final String SORT_ACTION_COMMAND = "sort";
	private static final String SORT_BUTTON_LABEL = "Sort";

	// Sorting key labels for combo box
	private static final Object[] COMPARATOR_LABELS = {
			"Album, disc, track (ascending)",
			"Album, disc, track (descending)",
			"Artist, album, disc, track (ascending)",
			"Artist, album, disc, track (descending)", "Title (ascending)",
			"Title (descending)", "Path (ascending)", "Path (descending)" };

	// Sorting keys
	private static final SongComparator[] COMPARATORS = {
			ComparatorFactory.ASCENDING_ALBUM_DISC_TRACK_COMPARATOR,
			ComparatorFactory.DESCENDING_ALBUM_DISC_TRACK_COMPARATOR,
			ComparatorFactory.ASCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR,
			ComparatorFactory.DESCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR,
			ComparatorFactory.ASCENDING_TITLE_COMPARATOR,
			ComparatorFactory.DESCENDING_TITLE_COMPARATOR,
			ComparatorFactory.ASCENDING_PATH_COMPARATOR,
			ComparatorFactory.DESCENDING_PATH_COMPARATOR, };

	private JButton sortButton;
	private JComboBox sortKeyComboBox;

	private PlaylistComponent playlist;

	public COMP202PlaylistSortPanel(PlaylistComponent playlist) {
		this.playlist = playlist;

		this.sortKeyComboBox = new JComboBox(COMPARATOR_LABELS);

		this.sortButton = new JButton(SORT_BUTTON_LABEL);
		this.sortButton.setActionCommand(SORT_ACTION_COMMAND);
		this.sortButton.addActionListener(new SortButtonListener());

		this.setLayout(new FlowLayout(FlowLayout.CENTER, SPACING, SPACING));
		this.add(this.sortKeyComboBox);
		this.add(this.sortButton);
	}

	private class SortButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			playlist.sort(COMPARATORS[sortKeyComboBox.getSelectedIndex()]);
		}
	}

	public PlaylistComponent getPlaylist() {
		return playlist;
	}

	public void setPlaylist(PlaylistComponent playlist) {
		this.playlist = playlist;
	}
}