import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.NoSuchElementException;

import comp202.fall2007.a5.comparator.SongComparator;
import comp202.fall2007.a5.util.Song;

public class IteratedPlaylist extends Playlist {
	public static final int EMPTY_PLAYLIST = -1;

	private int current;

	public IteratedPlaylist() {
		super();
		this.current = EMPTY_PLAYLIST;
	}

	public IteratedPlaylist(Playlist playlist) {
		super(playlist);
		if (playlist instanceof IteratedPlaylist) {
			IteratedPlaylist iteratedPlaylist;
			iteratedPlaylist = (IteratedPlaylist) playlist;
			this.current = iteratedPlaylist.current;
		} else {
			this.current = playlist.isEmpty() ? EMPTY_PLAYLIST : 0;
		}
	}

	public void add(int index, Song song) {
		super.add(index, song);
		if (index < this.current) {
			this.current++;
		}
	}

	public boolean add(Song song) {
		boolean result;
		int oldSize;

		oldSize = getSize();
		result = super.add(song);
		if (result && oldSize == 0) {
			this.current = 0;
		}
		return result;
	}

	public boolean addAll(ArrayList<Song> songList) {
		boolean result;
		int oldSize;

		oldSize = this.getSize();
		result = super.addAll(songList);
		if (result && (oldSize == 0)) {
			this.current = 0;
		}
		return result;
	}

	public boolean addAll(Playlist playList) {
		boolean result;
		int oldSize;

		oldSize = this.getSize();
		result = super.addAll(playList);
		if (result && (oldSize == 0)) {
			this.current = 0;
		}
		return result;

	}

	public boolean addAll(Song[] songArray) {
		boolean result;
		int oldSize;

		oldSize = this.getSize();
		result = super.addAll(songArray);
		if (result && (oldSize == 0)) {
			this.current = 0;
		}
		return result;
	}

	public void clear() {
		super.clear();
		this.current = EMPTY_PLAYLIST;
	}

	public int getCurrentPosition() {
		return this.current;
	}

	public boolean hasNextTrack() {
		return ((!this.isEmpty()) && (this.current != (this.getSize() - 1)));
	}

	public boolean hasPreviousTrack() {
		return ((!this.isEmpty()) && (this.current != 0));
	}

	public ArrayList<String> load(File path, MusicCollection collection)
			throws IOException {
		ArrayList<String> badPaths;

		badPaths = super.load(path, collection);
		this.current = this.isEmpty() ? EMPTY_PLAYLIST : 0;
		return badPaths;
	}

	public void moveDown(int index, int numberPositions) {
		int destination;

		super.moveDown(index, numberPositions);

		destination = index + numberPositions;
		if (this.current == index) {
			this.current = destination;
		} else {
			if ((index < this.current) && (this.current <= destination)) {
				this.current--;
			}
		}
	}

	public void moveToNextTrack() throws NoSuchElementException {
		if ((this.isEmpty()) || (this.current == (this.getSize() - 1))) {
			throw new NoSuchElementException();
		}
		this.current++;
	}

	public void moveToPreviousTrack() throws NoSuchElementException {
		if ((this.isEmpty()) || (this.current == 0)) {
			throw new NoSuchElementException();
		}
		this.current--;
	}

	public void moveUp(int index, int numberPositions) {
		int destination;

		super.moveUp(index, numberPositions);

		destination = index - numberPositions;
		if (this.current == index) {
			this.current = destination;
		} else {
			if ((destination <= this.current) && (this.current < index)) {
				this.current++;
			}
		}
	}

	public Song remove(int index) {
		Song song;
		int size;

		size = this.getSize();
		song = super.remove(index);
		if (index == this.current) {
			if (size == 1) {
				this.current = EMPTY_PLAYLIST;
			} else {
				if (this.current == (size - 1)) {
					this.current--;
				}
			}
		} else {
			if (index < this.current) {
				this.current--;
			}
		}
		return song;
	}

	public void setCurrentPosition(int current)
			throws IndexOutOfBoundsException {
		if ((current < 0) || (current >= this.getSize())) {
			throw new IndexOutOfBoundsException();
		}
		this.current = current;
	}

	public void sort(SongComparator comparator) {
		Song song;

		song = this.get(this.current);
		super.sort(comparator);
		this.current = 0;
		while (this.get(this.current) != song) {
			this.current++;
		}
	}
}
