// Instructions: Place in the same directory as Playlist.java and
// MusicCollection.java. Compile and run.

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Scanner;

import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.util.Song;

public class MusicCollectionTest {
	private static Scanner keyboard = new Scanner(System.in);
	
	private static PrintStream out;

	private static final Song s1 = new Song(new File("foo.mp3"), "Joe Bleau",
			"Computer Science Songs", "Foo", 6, 4);
	private static final Song s2 = new Song(new File("boring-life.mp3"),
			"Jane Doe", "Generic Example Music", "My Boring Life", 4, 2);
	private static final Song s3 = new Song(new File("procrastinator.mp3"),
			"Mathieu Petitpas", "Grad Student Music", "I'm a Procrastinator",
			3, 3);
	private static final Song s4 = new Song(new File("concurrency.mp3"),
			"Clark Verbrugge", "SOCS Greatest Hits",
			"The Concurrency Complaint", 1, 5);
	private static final Song s5 = new Song(new File("random-walk.mp3"),
			"Ladan Mahabadi", "Grad Student Music", "Random Walk", 1, 5);

	private static String displaySong(Song song) {
		if (song == null) {
			return "null";
		}
		return "Song \"" + song.getTitle() + "\" by artist \""
				+ song.getArtist() + "\" on album \"" + song.getAlbum()
				+ "\" (Track #" + song.getTrackNumber() + "; length: "
				+ song.getLength() + "s; path: " + song.getFilePath() + ")";
	}

	private static final FilterFactory factory = new FilterFactory();

	private static final SongFilter allSongsFilter = factory
			.createEverythingFilter();

	private static void displayCollectionState(MusicCollection collection,
			String name) {
		Playlist contents;
		int contentsSize;

		contents = collection.getSongs(allSongsFilter);
		contentsSize = contents.getSize();

		out.println("Contents of collection " + name + ":");
		if (contents.isEmpty()) {
			out.println("NOTHING");
		} else {
			for (int i = 0; i < contentsSize; i++) {
				out.println("- "
						+ displaySong(contents.get(i)));
			}
		}

		if (out == System.out) {
			out.println("Press ENTER to continue");
			keyboard.nextLine();
		} else {
			out.println();
		}
	}

	private static MusicCollection testConstructor(String name) {
		MusicCollection collection;

		out.println("***** Testing: Constructor MusicCollection()");
		out.println();

		out.println("Test 1 / 1: Creating " + name
				+ " with default constructor...");
		collection = new MusicCollection();
		displayCollectionState(collection, name);
		return collection;
	}

	private static void testAdd(MusicCollection collection, String name) {
		out.println("***** Testing: add(Song song)");
		out.println();

		// Adding a valid song
		out.println("Test 1 / 4: Adding " + displaySong(s1)
				+ " to collection " + name + "...");
		collection.add(s1);
		displayCollectionState(collection, name);

		// Adding a different song
		out.println("Test 2 / 4: Adding " + displaySong(s2)
				+ " to collection " + name + "...");
		collection.add(s2);
		displayCollectionState(collection, name);

		// Adding a duplicate
		out.println("Test 3 / 4: Adding " + displaySong(s1)
				+ " to collection " + name + "...");
		collection.add(s1);
		displayCollectionState(collection, name);

		// Adding null
		out.println("Test 4 / 4: Adding " + displaySong(null)
				+ " to collection " + name + "...");
		collection.add(null);
		displayCollectionState(collection, name);
	}

	private static void testAddAllArray(MusicCollection collection, String name) {
		Song[] songArray = { s3, s1, null };

		out.println("***** Testing: addAll(Song[] songArray)");
		out.println();

		// Adding a valid song
		out.println("Test 1 / 1: Adding array of Song to collection "
				+ name + "...");
		out.println("Songs to be added: ");
		for (int i = 0; i < songArray.length; i++) {
			out.println(i + ": " + displaySong(songArray[i]));
		}
		collection.addAll(songArray);
		displayCollectionState(collection, name);
	}
	
	private static void testAddAllArrayList(MusicCollection collection, String name) {
		ArrayList<Song> songList;
		int songListSize;

		out.println("***** Testing: addAll(ArrayList<Song> songList)");
		out.println();
		
		songList = new ArrayList<Song>();
		songList.add(s4);
		songList.add(s5);
		songList.add(s1);
		songList.add(null);
		songListSize = songList.size();

		// Adding a valid song
		out.println("Test 1 / 1: Adding array of Song to collection "
				+ name + "...");
		out.println("Songs to be added: ");
		for (int i = 0; i < songListSize; i++) {
			out.println(i + ": " + displaySong(songList.get(i)));
		}
		collection.addAll(songList);
		displayCollectionState(collection, name);
	}
	
	private static void testGet(MusicCollection collection, String name) {
		Song song;
		File file;
		
		out.println("***** Testing: get(File path)");
		out.println();

		file = s1.getFilePath();
		out.println("Test 1 / 2: retrieving Song corresponding to " + file + " from collection " + name + "...");
		song = collection.getSong(file);
		if (song != null) {
			out.println("Retrieved: " + displaySong(song));
		} else {
			out.println("*** PROBLEM: null retrieved (expected " + displaySong(s1) + ")");
		}
		displayCollectionState(collection, name);

		file = new File("bar.mp3");
		out.println("Test 2 / 2: retrieving Song corresponding to " + file + " from collection " + name + "...");
		song = collection.getSong(file);
		if (song == null) {
			out.println("No song exists in collection " + name + " whose path is " + file);
		} else {
			out.println("*** PROBLEM: retrieved " + displaySong(s1) + "(expected null )");
		}
		displayCollectionState(collection, name);
	}
	
	private static void testClear(MusicCollection collection, String name) {
		out.println("***** Testing: clear()");
		out.println();

		out.println("Test 1 / 1: Clearing " + name + "...");
		out.println("Before clearing: ");
		displayCollectionState(collection, name);
		collection.clear();
		out.println("After clearing: ");
		displayCollectionState(collection, name);
		
	}

	private static void stopTest() {
		String answer;

		if (out == System.out) {
			System.out.print("Would you like to stop the test (y/Y to stop)? ");
			answer = keyboard.nextLine();
			if (answer.equalsIgnoreCase("y")) {
				System.exit(0);
			} else {
				System.out.println();
				System.out.println();
			}
		} else {
			out.println();
		}
	}

	public static void main(String[] args) {
		MusicCollection c;

		File resultsFile;
		String choice;
		boolean done;

		System.out.println("Where would you like the results to be displayed?");
		System.out.println("1. On the screen");
		System.out.println("2. To a file");
		System.out.print("Your choice (1 / 2): ");

		done = false;
		choice = null;
		while (!done) {
			choice = keyboard.nextLine();
			if (choice.equals("1") || choice.equals("2")) {
				done = true;
			} else {
				System.out
						.print("Invalid choice. Please enter again (1 / 2): ");
			}
		}
		System.out.println();
		System.out.println();

		if (choice.equals("1")) {
			out = System.out;
			resultsFile = null;
		} else {
			resultsFile = new File("test-results-" + System.currentTimeMillis()
					+ ".txt");
			try {
				out = new PrintStream(resultsFile);
			} catch (IOException exception) {
				out.println("Could not open " + resultsFile + " for writing");
				return;
			}
		}
		
		c = testConstructor("c");
		stopTest();
		testAdd(c, "c");
		stopTest();
		testAddAllArray(c, "c");
		stopTest();
		testAddAllArrayList(c, "c");
		stopTest();
		testGet(c, "c");
		stopTest();
		testClear(c, "c");
		
		System.out.println("All tests done"
				+ ((out != System.out) ? "; saved to file " + resultsFile : ""));
	}
}
