// Instructions: Place in the same directory as Playlist.java and
// MusicCollection.java. Compile and run.

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Random;
import java.util.Scanner;

import comp202.fall2007.a5.comparator.ComparatorFactory;
import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.util.Song;

public class PlaylistTest {
	private static Scanner keyboard = new Scanner(System.in);

	private static PrintStream out;

	private static final Song s1 = new Song(new File("foo.mp3"), "Joe Bleau",
			"Computer Science Songs", "Foo", 6, 4);
	private static final Song s2 = new Song(new File("boring-life.mp3"),
			"Jane Doe", "Generic Example Music", "My Boring Life", 4, 2);
	private static final Song s3 = new Song(new File("procrastinator.mp3"),
			"Mathieu Petitpas", "Grad Student Music", "I'm a Procrastinator",
			3, 3);
	private static final Song s4 = new Song(new File("concurrency.mp3"),
			"Clark Verbrugge", "SOCS Greatest Hits",
			"The Concurrency Complaint", 1, 5);
	private static final Song s5 = new Song(new File("random-walk.mp3"),
			"Ladan Mahabadi", "Grad Student Music", "Random Walk", 1, 5);

	private static File generateBadFile() {
		Random generator;
		String badFileName;
		File badFile;

		badFile = null;
		generator = new Random();
		while (badFile == null) {
			badFileName = "";

			for (int i = 0; i < 8; i++) {
				badFileName += (char) ('A' + generator.nextInt(26));
			}
			badFile = new File(badFileName);

			if (badFile.exists()) {
				badFile = null;
			}
		}
		return badFile;
	}

	private static String displaySong(Song song) {
		if (song == null) {
			return "null";
		}
		return "Song \"" + song.getTitle() + "\" by artist \""
				+ song.getArtist() + "\" on album \"" + song.getAlbum()
				+ "\" (Track #" + song.getTrackNumber() + "; length: "
				+ song.getLength() + "s; path: " + song.getFilePath() + ")";
	}

	private static void displayPlaylistState(Playlist list, String name,
			int expectedSize, boolean expectedEmpty) {
		int realSize;

		realSize = list.getSize();
		out.println("Contents of playlist " + name + ":");
		if (list.isEmpty()) {
			out.println("NONE");
		} else {
			for (int i = 0; i < realSize; i++) {
				out.println("- " + i + ": " + displaySong(list.get(i)));
			}
		}
		realSize = list.getSize();
		out.println(((realSize == expectedSize) ? "" : "*** PROBLEM: ")
				+ "Size of " + name + ": " + list.getSize() + " (expected: "
				+ expectedSize + ")");
		out.println(((list.isEmpty() == expectedEmpty) ? "" : "*** PROBLEM: ")
				+ "Playlist " + name + " is empty: " + list.isEmpty()
				+ " (expected: " + expectedEmpty + ")");
		if (out == System.out) {
			out.println("Press ENTER to continue");
			keyboard.nextLine();
		} else {
			out.println();
		}
	}

	private static Playlist testDefaultConstructor(String name) {
		Playlist list;

		out.println("***** Testing: Constructor Playlist()");
		out.println();

		out.println("Test 1 / 1: Creating " + name
				+ " with default constructor...");
		list = new Playlist();
		displayPlaylistState(list, name, 0, true);
		return list;
	}

	private static Playlist testCopyConstructor(Playlist originalList,
			String originalListName, String newListName) {
		Playlist emptyList;
		Playlist newList;

		out.println("***** Testing: Constructor Playlist(Playlist playlist)");
		out.println();

		emptyList = new Playlist();

		// Pass empty Playlist
		out
				.println("Test 1 / 4: Creating "
						+ newListName
						+ " with constructor that takes Playlist from Playlist emptyList...");
		displayPlaylistState(emptyList, "emptyList", 0, true);
		newList = new Playlist(emptyList);
		displayPlaylistState(newList, newListName, emptyList.getSize(),
				emptyList.isEmpty());

		// Pass non-empty Playlist
		out.println("Test 2 / 4: Creating " + newListName
				+ " with constructor that takes Playlist from Playlist "
				+ originalListName + "...");
		displayPlaylistState(originalList, originalListName, originalList
				.getSize(), originalList.isEmpty());
		newList = new Playlist(originalList);
		displayPlaylistState(newList, newListName, originalList.getSize(),
				originalList.isEmpty());

		// Change original Playlist
		out
				.println("Test 3 / 4: Removing first element from original Playlist "
						+ originalListName + "...");
		originalList.remove(0);
		displayPlaylistState(originalList, originalListName, originalList
				.getSize(), originalList.isEmpty());
		displayPlaylistState(newList, newListName, newList.getSize(), newList
				.isEmpty());

		// Change new Playlist
		out.println("Test 4 / 4: Removing last element from new Playlist "
				+ newListName + "...");
		newList.remove(newList.getSize() - 1);
		displayPlaylistState(originalList, originalListName, originalList
				.getSize(), originalList.isEmpty());
		displayPlaylistState(newList, newListName, newList.getSize(), newList
				.isEmpty());

		return newList;
	}

	private static void testAddAtEnd(Playlist list, String name) {
		boolean result;
		int expectedSize;

		out.println("***** Testing: add(Song song)");
		out.println();

		expectedSize = list.getSize();

		// Adding a valid song
		out.println("Test 1 / 3: Adding " + displaySong(s1)
				+ " to the end of playlist " + name + "...");
		result = list.add(s1);
		expectedSize++;
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of add() operation: " + result + " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding the same song again
		out.println("Test 2 / 3: Adding " + displaySong(s1)
				+ " again to the end of playlist " + name + "...");
		result = list.add(s1);
		expectedSize++;
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of add() operation: " + result + " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding null
		out.println("Test 3 / 3: Adding null to the end of playlist " + name
				+ "...");
		result = list.add(null);
		out
				.println((!result ? "" : "*** PROBLEM: ")
						+ "Result of add() operation: " + result
						+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testAddAnywhere(Playlist list, String name) {
		int expectedSize;

		out.println("***** Testing: add(int index, Song song)");
		out.println();

		expectedSize = list.getSize();

		// Adding to position 0
		out.println("Test 1 / 5: Adding " + displaySong(s2)
				+ " to position 0 of playlist " + name + "...");
		try {
			expectedSize++;
			list.add(0, s2);
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Adding to position equal to size of list
		out.println("Test 2 / 5: Adding " + displaySong(s2)
				+ " to position 3 of playlist " + name + "...");
		try {
			list.add(expectedSize, s2);
			expectedSize++;
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Adding to position equal to size of list - 1
		out.println("Test 3 / 5: Adding " + displaySong(s3)
				+ " to position 3 of playlist " + name + "...");
		try {
			list.add(expectedSize - 1, s3);
			expectedSize++;
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Adding to position -1
		out.println("Test 4 / 5: Adding " + displaySong(s3)
				+ " to position -1 of playlist " + name + "...");
		try {
			list.add(-1, s3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Adding to position expectedSize + 1
		out.println("Test 5 / 5: Adding " + displaySong(s3)
				+ " to position 6 of playlist " + name + "...");
		try {
			list.add(expectedSize + 1, s3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testAddAllArray(Playlist list, String name) {
		Song[] nullArray = null;
		Song[] allGoodArray = { s1, s4, s5 };
		Song[] someNullArray = { s2, null, s3 };
		Song[] allNullArray = { null, null, null };
		Song[] lengthZeroArray = {};
		boolean result;
		int expectedSize;

		out.println("***** Testing: addAll(Song[] songArray)");
		out.println();

		expectedSize = list.getSize();

		// Adding array full of good songs
		out.println("Test 1 / 5: Adding array full of songs to playlist "
				+ name + "...");
		out.println("Array contents: ");
		for (int i = 0; i < allGoodArray.length; i++) {
			out.println(i + ": " + displaySong(allGoodArray[i]));
		}
		result = list.addAll(allGoodArray);
		expectedSize += allGoodArray.length;
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding array with some null elements
		out
				.println("Test 2 / 5: Adding array with some null elements to playlist "
						+ name + "...");
		out.println("Array contents: ");
		for (int i = 0; i < someNullArray.length; i++) {
			out.println(i + ": " + displaySong(someNullArray[i]));
		}
		result = list.addAll(someNullArray);
		expectedSize += 2;
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding array with all null elements
		out
				.println("Test 3 / 5: Adding array with all null elements to playlist "
						+ name + "...");
		result = list.addAll(allNullArray);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding array with length 0
		out.println("Test 4 / 5: Adding array of length 0 to playlist " + name
				+ "...");
		result = list.addAll(lengthZeroArray);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding null array
		out
				.println("Test 5 / 5: Adding null array to playlist " + name
						+ "...");
		result = list.addAll(nullArray);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testAddAllArrayList(Playlist list, String name) {
		ArrayList<Song> allGoodList;
		ArrayList<Song> someNullList;
		ArrayList<Song> allNullList;
		ArrayList<Song> lengthZeroList;
		boolean result;
		int expectedSize;

		allGoodList = new ArrayList<Song>();
		allGoodList.add(s1);
		allGoodList.add(s4);
		allGoodList.add(s5);

		someNullList = new ArrayList<Song>();
		someNullList.add(s2);
		someNullList.add(null);
		someNullList.add(s3);

		allNullList = new ArrayList<Song>();
		allNullList.add(null);
		allNullList.add(null);
		allNullList.add(null);

		lengthZeroList = new ArrayList<Song>();

		out.println("***** Testing: addAll(ArrayList<Song> songList)");
		out.println();

		expectedSize = list.getSize();

		// Adding ArrayList full of good songs
		out.println("Test 1 / 4: Adding ArrayList full of songs to playlist "
				+ name + "...");
		out.println("Array contents: ");
		for (int i = 0; i < allGoodList.size(); i++) {
			out.println(i + ": " + displaySong(allGoodList.get(i)));
		}
		result = list.addAll(allGoodList);
		expectedSize += allGoodList.size();
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding ArrayList with some null elements
		out
				.println("Test 2 / 4: Adding ArrayList with some null elements to playlist "
						+ name + "...");
		for (int i = 0; i < someNullList.size(); i++) {
			out.println(i + ": " + displaySong(someNullList.get(i)));
		}
		result = list.addAll(someNullList);
		expectedSize += 2;
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: true)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding ArrayList with all null elements
		out
				.println("Test 3 / 4: Adding ArrayList with all null elements to playlist "
						+ name + "...");
		result = list.addAll(allNullList);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);

		// Adding ArrayList with length 0
		out.println("Test 4 / 4: Adding array of length 0 to playlist " + name
				+ "...");
		result = list.addAll(lengthZeroList);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testAddAllPlaylist(Playlist list1, String name1,
			Playlist list2, String name2) {
		Playlist emptyPlaylist;
		boolean result;
		int size1;
		int size2;

		out.println("***** Testing: addAll(Playlist playlist)");
		out.println();

		size1 = list1.getSize();
		size2 = list2.getSize();
		emptyPlaylist = new Playlist();

		// Adding empty Playlist
		out.println("Test 1 / 4: Adding empty playlist to playlist " + name1
				+ "...");
		result = list1.addAll(emptyPlaylist);
		out.println((!result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: false)");
		displayPlaylistState(list1, name1, size1, false);

		// Adding non-empty PlayList
		out.println("Test 2 / 4: Adding playlist " + name2 + " to playlist "
				+ name1 + "...");
		result = list1.addAll(list2);
		out.println((result ? "" : "*** PROBLEM: ")
				+ "Result of addAll() operation: " + result
				+ " (expected: true)");
		displayPlaylistState(list1, name1, size1 + size2, false);

		// Changing first playlist
		out.println("Test 3 / 4: Removing first song from playlist " + name1
				+ "...");
		list1.remove(0);
		displayPlaylistState(list1, name1, size1 + size2 - 1, false);
		displayPlaylistState(list2, name2, size2, false);

		out.println("Test 4 / 4: Removing last song from playlist " + name2
				+ "...");
		list2.remove(size2 - 1);
		displayPlaylistState(list1, name1, size1 + size2 - 1, false);
		displayPlaylistState(list2, name2, size2 - 1, false);
	}

	private static void testGet(Playlist list, String name) {
		Song song;
		int expectedSize;

		out.println("***** Testing: get(int index)");
		out.println();

		expectedSize = list.getSize();

		// Getting song at position 0
		out.println("Test 1 / 5: Retrieving song at position 0 from playlist "
				+ name + "...");
		try {
			song = list.get(0);
			out.println("Retrieved: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize - 1)
		out.println("Test 2 / 5: Retrieving song at position "
				+ (expectedSize - 1) + " from playlist " + name + "...");
		try {
			song = list.get(expectedSize - 1);
			out.println("Retrieved: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize / 2)
		out.println("Test 3 / 5: Retrieving song at position "
				+ (expectedSize / 2) + " from playlist " + name + "...");
		try {
			song = list.get(expectedSize / 2);
			out.println("Retrieved: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position -1
		out.println("Test 4 / 5: Retrieving song at position -1 from playlist "
				+ name + "...");
		try {
			song = list.get(-1);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
			out.println("Retrieved: " + displaySong(song));
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize)
		out.println("Test 5 / 5: Retrieving song at position " + expectedSize
				+ " from playlist " + name + "...");
		try {
			song = list.get(expectedSize);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
			out.println("Retrieved: " + displaySong(song));
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testRemove(Playlist list, String name) {
		Song song;
		int expectedSize;

		out.println("***** Testing: remove(int index)");
		out.println();

		expectedSize = list.getSize();

		// Getting song at position 0
		out.println("Test 1 / 5: Removing song at position 0 from playlist "
				+ name + "...");
		try {
			song = list.remove(0);
			expectedSize--;
			out.println("Rmoved: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize - 1)
		out.println("Test 2 / 5: Removing song at position "
				+ (expectedSize - 1) + " from playlist " + name + "...");
		try {
			song = list.remove(expectedSize - 1);
			expectedSize--;
			out.println("Removed: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize / 2)
		out.println("Test 3 / 5: Removing song at position "
				+ (expectedSize / 2) + " from playlist " + name + "...");
		try {
			song = list.remove(expectedSize / 2);
			expectedSize--;
			out.println("Removed: " + displaySong(song));
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position -1
		out.println("Test 4 / 5: Removing song at position -1 from playlist "
				+ name + "...");
		try {
			song = list.get(-1);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
			out.println("Removed: " + displaySong(song));
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Getting song at position (expectedSize)
		out.println("Test 5 / 5: Removing song at position " + expectedSize
				+ " from playlist " + name + "...");
		try {
			song = list.get(expectedSize);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
			out.println("Removed: " + displaySong(song));
		} catch (IndexOutOfBoundsException exception) {
			out.println("IndexOutOfBoundsException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);

	}

	private static void testMoveUp(Playlist list, String name) {
		int expectedSize;

		out.println("***** Testing: moveUp(int index, int numberPositions)");
		out.println();

		expectedSize = list.getSize();

		// Moving middle song 3 positions
		out.println("Test 1 / 4: Moving song at position " + (expectedSize / 2)
				+ " from playlist " + name + " up 3 positions...");
		try {
			list.moveUp(expectedSize / 2, 3);
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: IllegalArgumentException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Original position out of bounds
		out.println("Test 2 / 4: Moving song at position -1 from playlist "
				+ name + " up 3 positions...");
		try {
			list.moveUp(-1, 3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("An IndexOutOfBoundsException was thrown (expected one)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: An IllegalArgumentException was thrown (expected IndexOutOfBoundsException)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Destination position out of bounds
		out.println("Test 3 / 4: Moving song at position 2 from playlist "
				+ name + " up 3 positions...");
		try {
			list.moveUp(2, 3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("An IndexOutOfBoundsException was thrown (expected one)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: An IllegalArgumentException was thrown (expected IndexOutOfBoundsException)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Number positions invalid
		out.println("Test 4 / 4: Moving song at position 2 from playlist "
				+ name + " up 0 positions...");
		try {
			list.moveUp(2, 0);
			out
					.println("*** PROBLEM: No exception was thrown (expected IllegalArgumentException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: An InedxOutOfBoundsException was thrown (expected IllegalArgumentException)");
		} catch (IllegalArgumentException exception) {
			out
					.println("An IllegalArgumentException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);

	}

	private static void testMoveDown(Playlist list, String name) {
		int expectedSize;

		out.println("***** Testing: moveDown(int index, int numberPositions)");
		out.println();

		expectedSize = list.getSize();

		// Moving middle song 3 positions
		out.println("Test 1 / 4: Moving song at position " + (expectedSize / 2)
				+ " from playlist " + name + " down 3 positions...");
		try {
			list.moveDown(expectedSize / 2, 3);
			out.println("No exception was thrown (expected none)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: IndexOutOfBoundsException was thrown (expected none)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: IllegalArgumentException was thrown (expected none)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Original position out of bounds
		out.println("Test 2 / 4: Moving song at position -1 from playlist "
				+ name + " down 3 positions...");
		try {
			list.moveDown(-1, 3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("An IndexOutOfBoundsException was thrown (expected one)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: An IllegalArgumentException was thrown (expected IndexOutOfBoundsException)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Destination position out of bounds
		out.println("Test 3 / 4: Moving song at position " + (expectedSize - 2)
				+ " from playlist " + name + " down 3 positions...");
		try {
			list.moveDown(expectedSize - 2, 3);
			out
					.println("*** PROBLEM: No exception was thrown (expected IndexOutOfBoundsException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("An IndexOutOfBoundsException was thrown (expected one)");
		} catch (IllegalArgumentException exception) {
			out
					.println("*** PROBLEM: An IllegalArgumentException was thrown (expected IndexOutOfBoundsException)");
		}
		displayPlaylistState(list, name, expectedSize, false);

		// Number positions invalid
		out.println("Test 4 / 4: Moving song at position " + (expectedSize - 2)
				+ " from playlist " + name + " down 0 positions...");
		try {
			list.moveDown(expectedSize - 2, 0);
			out
					.println("*** PROBLEM: No exception was thrown (expected IllegalArgumentException)");
		} catch (IndexOutOfBoundsException exception) {
			out
					.println("*** PROBLEM: An InedxOutOfBoundsException was thrown (expected IllegalArgumentException)");
		} catch (IllegalArgumentException exception) {
			out
					.println("An IllegalArgumentException was thrown (expected one)");
		}
		displayPlaylistState(list, name, expectedSize, false);
	}

	private static void testClear(Playlist list, String name) {
		out.println("***** Testing: clear()");

		out.println("Test 1 / 1: Clearing " + name + "...");
		out.println("Before clearing: ");
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());
		list.clear();
		out.println("After clearing: ");
		displayPlaylistState(list, name, 0, true);
	}

	private static void testSort(Playlist list, String name) {
		boolean originalEmpty;
		int originalSize;

		originalEmpty = list.isEmpty();
		originalSize = list.getSize();

		out.println("*** Testing sort()");
		out.println();

		out.println("Original list " + name + ":");
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out
				.println("Test 1 / 16: Sorting " + name
						+ " by title (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_TITLE_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 2 / 16: Sorting " + name
				+ " by title (descending)...");
		list.sort(ComparatorFactory.DESCENDING_TITLE_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 3 / 16: Sorting " + name
				+ " by artist (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_ARTIST_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 4 / 16: Sorting " + name
				+ " by artist (descending)...");
		list.sort(ComparatorFactory.DESCENDING_ARTIST_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out
				.println("Test 5 / 16: Sorting " + name
						+ " by album (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_ALBUM_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 6 / 16: Sorting " + name
				+ " by album (descending)...");
		list.sort(ComparatorFactory.DESCENDING_ALBUM_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out
				.println("Test 7 / 16: Sorting " + name
						+ " by track (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 8 / 16: Sorting " + name
				+ " by track (descending)...");
		list.sort(ComparatorFactory.DESCENDING_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 9 / 16: Sorting " + name
				+ " by length (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_LENGTH_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 10 / 16: Sorting " + name
				+ " by length (descending)...");
		list.sort(ComparatorFactory.DESCENDING_LENGTH_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out
				.println("Test 11 / 16: Sorting " + name
						+ " by path (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_PATH_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 12 / 16: Sorting " + name
				+ " by path (descending)...");
		list.sort(ComparatorFactory.DESCENDING_PATH_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 13 / 16: Sorting " + name
				+ " by album, disc, track (ascending)...");
		list.sort(ComparatorFactory.ASCENDING_ALBUM_DISC_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 14 / 16: Sorting " + name
				+ " by album, disc, track (descending)...");
		list.sort(ComparatorFactory.DESCENDING_ALBUM_DISC_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 15 / 16: Sorting " + name
				+ " by artist, album, disc, track (ascending)...");
		list
				.sort(ComparatorFactory.ASCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);

		out.println("Test 16 / 16: Sorting " + name
				+ " by artist, album, disc, track (descending)...");
		list
				.sort(ComparatorFactory.DESCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR);
		displayPlaylistState(list, name, originalSize, originalEmpty);
	}

	private static File testSave(Playlist list, String name) {
		Random generator;
		File outputFile;
		File badFile;
		boolean originalIsEmpty;
		int originalSize;
		int currentSize;

		out.println("*** Testing save(File path)");
		out.println();

		originalSize = list.getSize();
		originalIsEmpty = list.isEmpty();

		generator = new Random();
		outputFile = null;
		while (outputFile == null) {
			outputFile = new File("test-playlistfile-" + generator.nextInt(100)
					+ ".txt");
			if (outputFile.exists()) {
				outputFile = null;
			}
		}

		badFile = new File(generateBadFile(), outputFile.getPath());

		out.println("Test 1 / 2: Saving playlist " + name
				+ " to unwritable file " + badFile.getAbsolutePath() + "...");
		try {
			list.save(badFile);
			out
					.println("*** PROBLEM: No exception was thrown (expected IOException)");
		} catch (IOException exception) {
			out.println("IOException thrown (one expected)");
		}
		displayPlaylistState(list, name, originalSize, originalIsEmpty);

		out.println("Test 2 / 2: Saving playlist " + name
				+ " to writable file " + outputFile.getAbsolutePath() + "...");
		try {
			list.save(outputFile);
			out.println("No exception thrown (none expected)");
			out.println("Check file " + outputFile.getAbsolutePath()
					+ "; contents should be: ");
			currentSize = list.getSize();
			for (int i = 0; i < currentSize; i++) {
				out.println(list.get(i).getFilePath().getAbsolutePath());
			}
			out.println("(end of file)");
		} catch (IOException exception) {
			out.println("PROBLEM: IOException thrown (none expected)");
		}
		displayPlaylistState(list, name, originalSize, originalIsEmpty);

		return outputFile;
	}

	private static void testLoad(Playlist list, String name, File inputFile) {
		ArrayList<String> badFiles;
		Song[] someSongs = { s3, s4, s5 };
		Song[] allSongs = { s1, s2, s3, s4, s5 };
		FilterFactory factory;
		MusicCollection partialCollection;
		MusicCollection fullCollection;
		File badFile;
		boolean originalIsEmpty;
		int originalSize;

		out.println("*** Testing load(File path, MusicCollection collection)");
		out.println();

		factory = new FilterFactory();
		partialCollection = new MusicCollection();
		fullCollection = new MusicCollection();

		originalSize = list.getSize();
		originalIsEmpty = list.isEmpty();

		badFile = generateBadFile();
		out.println("Test 1 / 3: loading playlist " + name
				+ " from nonexistent file " + badFile + "...");
		displayPlaylistState(list, name, originalSize, originalIsEmpty);
		out.println("Attempting to load...");
		try {
			badFiles = list.load(badFile, partialCollection);
			out
					.println("*** PROBLEM: No exception thrown (IOException expected");
		} catch (IOException exception) {
			out.println("IOException thrown (one expected)");
		}
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());

		out.println("Test 2 / 3: loading playlist " + name
				+ " from input file " + inputFile + "...");
		partialCollection.addAll(someSongs);
		out.println("Using partial collection: ");
		displayPlaylistState(partialCollection.getSongs(factory
				.createEverythingFilter()), "partialCollection", 3, false);
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());
		try {
			badFiles = list.load(inputFile, partialCollection);
			out.println("No exception thrown (none expected)");
			if (badFiles.size() != 0) {
				out.println("Files which could not be loaded: ");
			} else {
				out
						.println("*** PROBLEM: List of unloadable files is empty, expected 2 unloadable files");
			}
			for (String path : badFiles) {
				out.println("- " + path);
			}
		} catch (IOException exception) {
			out.println("*** PROBLEM: IOException thrown (none expected)");
		}
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());

		out.println("Test 3 / 3: loading playlist " + name
				+ " from input file " + inputFile + "...");
		fullCollection.addAll(allSongs);
		out.println("Using full collection: ");
		displayPlaylistState(fullCollection.getSongs(factory
				.createEverythingFilter()), "fullCollection", 5, false);
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());
		try {
			badFiles = list.load(inputFile, fullCollection);
			out.println("No exception thrown (none expected)");
			if (badFiles.size() == 0) {
				out.println("All files successfully loaded");
			} else {
				out
						.println("*** PROBLEM: List of unloadable files is not empty, expected no unloadable files");
			}
			for (String path : badFiles) {
				out.println("- " + path);
			}
		} catch (IOException exception) {
			out.println("*** PROBLEM: IOException thrown (none expected)");
		}
		displayPlaylistState(list, name, list.getSize(), list.isEmpty());
	}

	private static void stopTest() {
		String answer;

		if (out == System.out) {
			System.out.print("Would you like to stop the test (y/Y to stop)? ");
			answer = keyboard.nextLine();
			if (answer.equalsIgnoreCase("y")) {
				System.exit(0);
			} else {
				System.out.println();
				System.out.println();
			}
		} else {
			out.println();
		}
	}

	public static void main(String[] args) {
		Playlist p1;
		Playlist p2;
		File file;
		File resultsFile;
		String choice;
		boolean done;

		System.out.println("Where would you like the results to be displayed?");
		System.out.println("1. On the screen");
		System.out.println("2. To a file");
		System.out.print("Your choice (1 / 2): ");

		done = false;
		choice = null;
		while (!done) {
			choice = keyboard.nextLine();
			if (choice.equals("1") || choice.equals("2")) {
				done = true;
			} else {
				System.out
						.print("Invalid choice. Please enter again (1 / 2): ");
			}
		}
		System.out.println();
		System.out.println();

		if (choice.equals("1")) {
			out = System.out;
			resultsFile = null;
		} else {
			resultsFile = new File("test-results-" + System.currentTimeMillis()
					+ ".txt");
			try {
				out = new PrintStream(resultsFile);
			} catch (IOException exception) {
				out.println("Could not open " + resultsFile + " for writing");
				return;
			}
		}

		p1 = testDefaultConstructor("p1");
		stopTest();
		testAddAtEnd(p1, "p1");
		stopTest();
		testAddAnywhere(p1, "p1");
		stopTest();
		testAddAllArray(p1, "p1");
		stopTest();
		testAddAllArrayList(p1, "p1");
		stopTest();
		testGet(p1, "p1");
		stopTest();
		testRemove(p1, "p1");
		stopTest();
		testMoveUp(p1, "p1");
		stopTest();
		testMoveDown(p1, "p1");
		stopTest();
		p2 = testCopyConstructor(p1, "p1", "p2");
		stopTest();
		testAddAllPlaylist(p1, "p1", p2, "p2");
		stopTest();
		testClear(p2, "p2");
		stopTest();
		file = testSave(p1, "p1");
		stopTest();
		testLoad(p1, "p1", file);
		stopTest();
		testSort(p1, "p1");
		System.out.println("All tests done"
				+ ((out != System.out) ? "; saved to file " + resultsFile : ""));
	}
}
