import java.io.File;
import java.util.ArrayList;
import java.util.Scanner;

import comp202.fall2007.a5.comparator.ComparatorFactory;
import comp202.fall2007.a5.comparator.SongComparator;
import comp202.fall2007.a5.filter.FilterFactory;
import comp202.fall2007.a5.filter.NumberMatchType;
import comp202.fall2007.a5.filter.SongFilter;
import comp202.fall2007.a5.loader.ConsoleDisplayListener;
import comp202.fall2007.a5.loader.SongLoader;
import comp202.fall2007.a5.util.Song;

public class SimpleMusicCollectionTest {
	public static void main(String[] args) {
		MusicCollectionBuilder builder;
		ConsoleDisplayListener consoleDisplay;
		MusicCollection collection;
		Scanner keyboard;
		SongLoader loader;
		String path;
		File rootDirectory;

		// Set up keyboard input
		keyboard = new Scanner(System.in);

		// Ask the user to enter the root directory where his / her MP3 files
		// are located
		System.out.print("Enter the directory where your music files are "
				+ "stored: ");
		path = keyboard.nextLine();
		rootDirectory = new File(path);

		// Load the files from the disk
		builder = new MusicCollectionBuilder();
		consoleDisplay = new ConsoleDisplayListener(rootDirectory);
		
		loader = new SongLoader();
		loader.addListener(builder);
		loader.addListener(consoleDisplay);
		loader.load(rootDirectory);

		// Retrieve the library
		collection = builder.getCollection();

		// Perform some tests
		ArrayList<String> allArtists;
		ArrayList<String> albums;
		FilterFactory factory;
		SongFilter filter;
		SongComparator sortKey;
		Playlist songs;
		int playlistSize;

		// Display the contents of the collection by artist, album, and track.
		allArtists = collection.getAllArtists();
		System.out.println();
		System.out.println("All artists: ");
		for (String artistName : allArtists) {
			System.out.println("- " + artistName);
			albums = collection.getContributedAlbums(artistName);

			for (String albumTitle : albums) {
				System.out.println("\t- " + albumTitle);
				songs = collection.getSongs(artistName, albumTitle);
				playlistSize = songs.getSize();

				for (int i = 0; i < playlistSize; i++) {
					Song song = songs.get(i);
					System.out.println("\t\t- " + song.getTrackNumber() + ". "
							+ song.getTitle());
				}
			}
		}

		// Display the contents of the collection by album and track.
		albums = collection.getAllAlbums();
		System.out.println();
		System.out.println("All albums: ");
		for (String albumTitle : albums) {
			System.out.println("- " + albumTitle);
			songs = collection.getSongs(albumTitle);
			playlistSize = songs.getSize();
			for (int i = 0; i < playlistSize; i++) {
				Song song = songs.get(i);
				System.out.println("\t- " + song.getTrackNumber() + ". "
						+ song.getTitle());
			}
		}

		// Retrieve all Songs whose track attribute is at least 5 and at most
		// 10.
		// Sort by album, by disc, and by track, all in descending order.
		factory = new FilterFactory();
		sortKey = ComparatorFactory.DESCENDING_ALBUM_DISC_TRACK_COMPARATOR;
		filter = factory.createAndFilter(new SongFilter[] {
				factory.createTrackFilter(5, NumberMatchType.AT_LEAST),
				factory.createTrackFilter(10, NumberMatchType.AT_MOST) });

		songs = collection.getSongs(filter);
		songs.sort(sortKey);
		playlistSize = songs.getSize();

		System.out.println();
		System.out.println("Filter: " + filter);
		System.out.println("Sort key: " + sortKey);
		for (int i = 0; i < playlistSize; i++) {
			Song song = songs.get(i);
			System.out.println("\t- " + song.getTrackNumber() + ". "
					+ song.getTitle());
		}
	}
}