package comp202.fall2007.a5.comparator;

import java.util.ArrayList;

/**
 * A <code>ComparatorFactory</code> creates <code>SongComparator</code>
 * objects that define certain orders on <code>Song</code>s based on their
 * attributes.
 */
public class ComparatorFactory {
	// *** Artist Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the
	 * artist name attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its artist name attribute appears lexicographically <b>before</b> the
	 * value of the artist name attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator ASCENDING_ARTIST_COMPARATOR = new ArtistComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the
	 * artist name attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its artist name attribute appears lexicographically <b>after</b> the
	 * value of the artist name attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator DESCENDING_ARTIST_COMPARATOR = new ArtistComparator(
			ComparatorOrder.DESCENDING);

	// *** Album Comparators

	/**
	 * A <code>SongComparator</code> which defines an order based on the album
	 * title attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its album title attribute appears lexicographically <b>before</b> the
	 * value of the album title attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator ASCENDING_ALBUM_COMPARATOR = new AlbumComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the album
	 * title attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its album title attribute appears lexicographically </b>after</b> the
	 * value of the album title attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator DESCENDING_ALBUM_COMPARATOR = new AlbumComparator(
			ComparatorOrder.DESCENDING);

	// *** Disc Number Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the disc
	 * number attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its disc number attribute is <b>less than</b> the value of the disc
	 * number attribute of the other <code>Song</code>.
	 */
	public static final SongComparator ASCENDING_DISC_COMPARATOR = new DiscComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the disc
	 * number attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its disc number attribute is <b>greater than</b> the value of the disc
	 * number attribute of the other <code>Song</code>.
	 */
	public static final SongComparator DESCENDING_DISC_COMPARATOR = new DiscComparator(
			ComparatorOrder.DESCENDING);

	// *** Track Number Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the track
	 * number attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its track number attribute is <b>less than</b> the value of the track
	 * number attribute of the other <code>Song</code>.
	 */
	public static final SongComparator ASCENDING_TRACK_COMPARATOR = new TrackComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the track
	 * number attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its track number attribute is <b>greater than</b> the value of the track
	 * number attribute of the other <code>Song</code>.
	 */
	public static final SongComparator DESCENDING_TRACK_COMPARATOR = new TrackComparator(
			ComparatorOrder.DESCENDING);

	// *** Path Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the path
	 * attributes of <code>Song</code>. In this order, a <code>Song</code>
	 * is defined to appear before another if the path of the file containing
	 * the corresponding audio track appears <b>before</b> the path of the file
	 * containing the audio track corresponding of the other <code>Song</code>,
	 * using the natural order defined by the <code>compareTo()</code> method
	 * of the <code>File</code> class.
	 */
	public static final SongComparator ASCENDING_PATH_COMPARATOR = new PathComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the path
	 * attributes of <code>Song</code>s. In this order, a <code>Song</code>
	 * is defined to appear before another if the path of the file containing
	 * the corresponding audio track appears <b>after</b> the path of the file
	 * containing the audio track corresponding to the other <code>Song</code>,
	 * using the natural order defined by the <code>compareTo()</code> method
	 * of the <code>File</code> class.
	 */
	public static final SongComparator DESCENDING_PATH_COMPARATOR = new PathComparator(
			ComparatorOrder.DESCENDING);

	// *** Track Title Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the track
	 * title attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its track title attribute appears lexicographically <b>before</b> the
	 * value of the track title attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator ASCENDING_TITLE_COMPARATOR = new TitleComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the track
	 * title attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its track title attribute appears lexicographically <b>after</b> the
	 * value of the track title attribute of the other <code>Song</code>,
	 * using a case-insensitive comparison.
	 */
	public static final SongComparator DESCENDING_TITLE_COMPARATOR = new TitleComparator(
			ComparatorOrder.DESCENDING);

	// *** Length Comparators ***

	/**
	 * A <code>SongComparator</code> which defines an order based on the
	 * length attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its length attribute is <b>less than</b> the value of the length
	 * attribute of the other <code>Song</code>.
	 */
	public static final SongComparator ASCENDING_LENGTH_COMPARATOR = new LengthComparator(
			ComparatorOrder.ASCENDING);

	/**
	 * A <code>SongComparator</code> which defines an order based on the
	 * length attributes of <code>Song</code>s. In this order, a
	 * <code>Song</code> is defined to appear before another if the value of
	 * its length attribute is <b>greater than</b> the value of the length
	 * attribute of the other <code>Song</cocde>.
	 */
	public static final SongComparator DESCENDING_LENGTH_COMPARATOR = new LengthComparator(
			ComparatorOrder.DESCENDING);

	// *** Disc-Track Combiners

	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>ASCENDING_DISC_COMPARATOR</code> and
	 * <code>ASCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until both have been tried without one of them determining that
	 * one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator ASCENDING_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { ASCENDING_DISC_COMPARATOR,
					ASCENDING_TRACK_COMPARATOR });

	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>DESCENDING_DISC_COMPARATOR</code> and
	 * <code>DESCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until both have been tried without one of them determining that
	 * one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator DESCENDING_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { DESCENDING_DISC_COMPARATOR,
					DESCENDING_TRACK_COMPARATOR });

	// *** Album-Disc-Track Combiners

	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>ASCENDING_ALBUM_COMPARATOR</code>,
	 * <code>ASCENDING_DISC_COMPARATOR</code>, and
	 * <code>ASCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until all three have been tried without one of them determining
	 * that one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator ASCENDING_ALBUM_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { ASCENDING_ALBUM_COMPARATOR,
					ASCENDING_DISC_COMPARATOR, ASCENDING_TRACK_COMPARATOR });
	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>DESCENDING_ALBUM_COMPARATOR</code>,
	 * <code>DESCENDING_DISC_COMPARATOR</code>, and
	 * <code>DESCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until all three have been tried without one of them determining
	 * that one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator DESCENDING_ALBUM_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { DESCENDING_ALBUM_COMPARATOR,
					DESCENDING_DISC_COMPARATOR, DESCENDING_TRACK_COMPARATOR });

	// *** Artist-Album-Disc-Track Combiners

	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>ASCENDING_ARTIST_COMPARATOR</code>,
	 * <code>ASCENDING_ALBUM_COMPARATOR</code>,
	 * <code>ASCENDING_DISC_COMPARATOR</code>, and
	 * <code>ASCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until all four have been tried without one of them determining
	 * that one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator ASCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { ASCENDING_ARTIST_COMPARATOR,
					ASCENDING_ALBUM_COMPARATOR, ASCENDING_DISC_COMPARATOR,
					ASCENDING_TRACK_COMPARATOR });
	/**
	 * A <code>SongComparator</code> which defines an order equivalent to
	 * applying <code>DESCENDING_ARTIST_COMPARATOR</code>,
	 * <code>DESCENDING_ALBUM_COMPARATOR</code>,
	 * <code>DESCENDING_DISC_COMPARATOR</code>, and
	 * <code>DESCENDING_TRACK_COMPARATOR</code>, in that order, until one of
	 * them determines that a <code>Song</code> occurs before or after the
	 * other, or until all four have been tried without one of them determining
	 * that one <code>Song</code> occurs before or after another</code>.
	 */
	public static final SongComparator DESCENDING_ARTIST_ALBUM_DISC_TRACK_COMPARATOR = new MultiComparator(
			new SongComparator[] { DESCENDING_ARTIST_COMPARATOR,
					DESCENDING_ALBUM_COMPARATOR, DESCENDING_DISC_COMPARATOR,
					DESCENDING_TRACK_COMPARATOR });

	/**
	 * Creates a new <code>ComparatorFactory</code>.
	 */
	public ComparatorFactory() {
	}

	/**
	 * <p>
	 * Creates a new <code>SongComparator</code> which defines an order that
	 * is the combination of the individual <code>SongComparator</code>s in
	 * the specified <code>ArrayList</code>.
	 * </p>
	 * 
	 * <p>
	 * The order defined by the returned <code>SongComparator</code> is
	 * determined by applying the <code>SongComparator</code>s in the
	 * specified <code>ArrayList</code>, in the order in which they appear in
	 * the specified <code>ArrayList</code>. This process is repeated until
	 * one of the <code>SongComparators</code> determines that a
	 * <code>Song</code> occurs before or after the other in the order it
	 * defines, or until all the <code>SongComparator</code> in the specified
	 * <code>ArrayList</code> have been tried without one determining that one
	 * of the <code>Song</code>s occurs before or after the other in the
	 * orders they define.
	 * </p>
	 * 
	 * <p>
	 * <code>null</code> elements contained in the specified
	 * <code>ArrayList</code> are not included in the new
	 * <code>SongComparator</code>. Subsequent changes to the specified
	 * <code>ArrayList</code> do not affect the <code>SongComparator</code>
	 * returned by this method.
	 * 
	 * @param subComparators
	 *            An <code>ArrayList</code> containing the individual
	 *            <code>SongComparator</code>s which are to be part of the
	 *            returned <code>SongComparator</code>.
	 * @return A <code>SongComparator</code> which is the combination of the
	 *         <code>SongComparator</code>s in <code>subComparators</code>.
	 * @throws NullPointerException
	 *             if <code>subComparators</code> is <code>null</code>.
	 */
	public SongComparator createMultiComparator(
			ArrayList<SongComparator> subComparators) {
		return new MultiComparator(subComparators);
	}

	/**
	 * <p>
	 * Creates a new <code>SongComparator</code> which defines an order that
	 * is the combination of the individual <code>SongComparator</code>s in
	 * the specified array.
	 * </p>
	 * 
	 * <p>
	 * The order defined by the returned <codeSongComparator</code> is
	 * determined by applying the <code>SongComparator</code>s in the
	 * specified array, in the order in which they appear in the specified
	 * array. This process is repeated until one of the <code>SongComparators</code>
	 * determines that a <code>Song</code> occurs before or after the other in
	 * the order it defines, or until all the <code>SongComparator</code> in
	 * the specified array have been tried without one determining that one of
	 * the <code>Song</code>s occurs before or after the other in the orders
	 * they define.
	 * </p>
	 * 
	 * <p>
	 * <code>null</code> elements contained in the specified array are not
	 * included in the new <code>SongComparator</code>. Subsequent changes to
	 * the specified array do not affect the <code>SongComparator</code>
	 * returned by this method.
	 * 
	 * @param subComparators
	 *            An array containing the individual <code>SongComparator</code>s
	 *            which are to be part of the returned <code>SongComparator</code>.
	 * @return A <code>SongComparator</code> which is the combination of the
	 *         <code>SongComparator</code> in <code>subComparators</code>.
	 * @throws NullPointerException
	 *             if <code>subComparators</code> is <code>null</code>.
	 */
	public SongComparator createMultiComparator(SongComparator[] subComparators) {
		return new MultiComparator(subComparators);
	}
}