package comp202.fall2007.a5.comparator;

import comp202.fall2007.a5.util.Song;

/**
 * A <code>SongComparator</code> object defines an order on <code>Song</code>s.
 * This order is <i>total</i>; that is, it is defined for any pair of
 * <code>Song</code>s. However, it is possible that this order is not
 * <i>strict</i>; that is, it is possible that for certain pairs of
 * <code>Song</code>s, neither <code>Song</code> appears before or after
 * the other. This situation occurs when the order defined by a
 * <code>SongComparator</code> is based on some attributes of
 * <code>Song</code>s, and the two <code>Song</code>s in the pair have
 * equal values for these attributes.
 */
public abstract class SongComparator {
	/**
	 * Creates a new <code>SongComparator</code>.
	 */
	public SongComparator() {
	}
	
	/**
	 * <p>
	 * Compares the two specified <code>Song</code>s and determines whether or
	 * not one occurs before the other in the order defined by this
	 * <code>SongComparator</code>.
	 * </p>
	 * 
	 * <p>
	 * If the first specified <code>Song</code> occurs before the second
	 * specified <code>Song</code> in the order defined by this
	 * <code>SongComparator</code>, this method returns a negative number. If
	 * the second specified <code>Song</code> occurs before the first
	 * specified <code>Song</code> in the order defined by this
	 * <code>SongComparator</code>, this method returns a positive number. If
	 * neither of the specified <code>Song</code> appears before the other in
	 * the order defined by this <code>SongComparator</code>, this method
	 * returns <code>0</code>.
	 * </p>
	 * 
	 * @param left
	 *            A <code>Song</code>.
	 * @param right
	 *            Another <code>Song</code>.
	 * @return A negative number, <code>0</code>, or a positive number if
	 *         <code>left</code> occurs before, neither before nor after, or
	 *         after <code>right</code>, respectively.
	 */
	public abstract int compareTo(Song left, Song right);

	/**
	 * Returns a <code>String</code> representation of this
	 * <code>SongComparator</code>, which describes the order it defines.
	 * 
	 * @return A <code>String</code> representation of this
	 *         <code>StringComparator</code>.
	 */
	public abstract String toString();
}
