package comp202.fall2007.a5.filter;

import java.io.File;
import java.util.ArrayList;

/**
 * <p>
 * A <code>FilterFactory</code> provides methods to create
 * <code>SongFilter</code>s that define various conditions based on the
 * attributes of <code>Song</code>s, or on the conditions defined by other
 * <code>SongFilter</code>s.
 * </p>
 */
public class FilterFactory {
	/**
	 * Creates a new <code>FilterFactory</code>.
	 */
	public FilterFactory() {
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose album attribute matches the specified
	 * <code>String</code>, using a match type defined by the specified
	 * <code>StringMatchType</code>.
	 * </p>
	 * 
	 * @param albumTitle
	 *            The album title used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s album attribute matches
	 *            <code>albumTitle</code>. The comparison is case-<b>IN</b>sensitive.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose album attribute matches
	 *         <code>albumTitle</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createAlbumFilter(String albumTitle,
			StringMatchType matchType) {
		return new AlbumFilter(albumTitle, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose artist attribute matches the specified
	 * <code>String</code>, using a match type defined by the specified
	 * <code>StringMatchType</code>.
	 * </p>
	 * 
	 * @param artistName
	 *            The artist name used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s artist attribute matches
	 *            <code>artistTitle</code>. The comparison is case-<b>IN</b>sensitive.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose artist attribute matches
	 *         <code>albumTitle</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createArtistFilter(String artistName,
			StringMatchType matchType) {
		return new ArtistFilter(artistName, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose title attribute matches the specified
	 * <code>String</code>, using a match type defined by the specified
	 * <code>StringMatchType</code>.
	 * </p>
	 * 
	 * @param songTitle
	 *            The song title used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s title attribute matches
	 *            <code>artistTitle</code>. The comparison is case-<b>IN</b>sensitive.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose title attribute matches
	 *         <code>albumTitle</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createTitleFilter(String songTitle,
			StringMatchType matchType) {
		return new TitleFilter(songTitle, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose path attribute is equal to the specified
	 * <code>File</code>, as defined by the equality of <code>File</code>
	 * objects.
	 * </p>
	 * 
	 * @param songPath
	 *            The path used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose path attribute is equal to
	 *         <code>songPath</code>.
	 */
	public SongFilter createPathFilter(File songPath) {
		return new PathFilter(songPath);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose length attribute matches the specified
	 * <code>String</code>, using a match type defined by the specified
	 * <code>NumberMatchType</code>.
	 * </p>
	 * 
	 * @param length
	 *            The length value used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s length matches <code>length</code>.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose length attribute matches
	 *         <code>length</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createLengthFilter(int length, NumberMatchType matchType) {
		return new LengthFilter(length, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose disc attribute matches the specified integer,
	 * using a match type defined by the specified <code>NumberMatchType</code>.
	 * </p>
	 * 
	 * @param discNumber
	 *            The disc number used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s disc number matches
	 *            <code>discNumber</code>.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose disc number attribute matches
	 *         <code>discNumber</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createDiscFilter(int discNumber, NumberMatchType matchType) {
		return new DiscFilter(discNumber, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts all
	 * <code>Song</code>s whose track number attribute matches the specified
	 * integer, using a match type defined by the specified
	 * <code>NumberMatchType</code>.
	 * </p>
	 * 
	 * @param trackNumber
	 *            The track number used in the condition defined by the returned
	 *            <code>SongFilter</code>.
	 * @param matchType
	 *            The type of comparison used by the returned
	 *            <code>SongFilter</code> to determine whether a
	 *            <code>Song</code>'s track number matches
	 *            <code>discNumber</code>.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s whose track number attribute matches
	 *         <code>trackNumber</code>, according to the type of comparison
	 *         defined by <code>matchType</code>.
	 */
	public SongFilter createTrackFilter(int trackNumber,
			NumberMatchType matchType) {
		return new TrackFilter(trackNumber, matchType);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which accepts every
	 * <code>Song</code>. That is, the returned <code>SongFilter</code>'s
	 * <code>accept()</code> method always returns <code>true</code>.
	 * </p>
	 * 
	 * @return A new <code>SongFilter</code> which accepts every
	 *         <code>Song</code>.
	 */
	public SongFilter createEverythingFilter() {
		return new EverythingFilter();
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which rejects every
	 * <code>Song</code>. That is, the returned <code>SongFilter</code>'s
	 * <code>accept()</code> method always returns <code>false</code>.
	 * </p>
	 * 
	 * @return A new <code>SongFilter</code> which rejects every
	 *         <code>Song</code>.
	 */
	public SongFilter createNothingFilter() {
		return new NothingFilter();
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> object which accepts a given
	 * <code>Song</code> if and only if all <code>SongFilter</code>s in the
	 * specified <code>ArrayList</code> accept the given <code>Song</code>.
	 * </p>
	 * 
	 * <p>
	 * Subsequent changes to the specified <code>ArrayList</code> do not
	 * affect the state of the <code>SongFilter</code> that this method
	 * creates and returns.
	 * </p>
	 * 
	 * @param subFilters
	 *            An <code>ArrayList</code> of <code>SongFilters</code>; to
	 *            be accepted by the newly-created <code>SongFilter</code>, a
	 *            <code>Song</code> must be accepted by all
	 *            <code>SongFilter</code>s in this <code>ArrayList</code>.
	 * @return A new <code>SongFilter</code> which accepts a given
	 *         <code>Song</code> if and only if all <code>SongFilters</code>
	 *         in <code>subFilters</code> accept the given <code>Song</code>.
	 */
	public SongFilter createAndFilter(ArrayList<SongFilter> subFilters) {
		return new AndFilter(subFilters);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> object which accepts a given
	 * <code>Song</code> if and only if all <code>SongFilter</code>s in the
	 * specified array accept the given <code>Song</code>.
	 * </p>
	 * 
	 * <p>
	 * Subsequent changes to the specified array do not affect the state of the
	 * <code>SongFilter</code> that this method creates and returns.
	 * </p>
	 * 
	 * @param subFilters
	 *            An array of <code>SongFilters</code>; to be accepted by the
	 *            newly-created <code>SongFilter</code>, a <code>Song</code>
	 *            must be accepted by all <code>SongFilter</code>s in this
	 *            array.
	 * @return A new <code>SongFilter</code> which accepts a given
	 *         <code>Song</code> if and only if all <code>SongFilters</code>
	 *         in <code>subFilters</code> accept the given <code>Song</code>.
	 */
	public SongFilter createAndFilter(SongFilter[] subFilters) {
		return new AndFilter(subFilters);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> object which accepts a given
	 * <code>Song</code> if and only if at least one of the
	 * <code>SongFilter</code>s in the specified <code>ArrayList</code>
	 * accepts the given <code>Song</code>.
	 * </p>
	 * 
	 * <p>
	 * Subsequent changes to the specified <code>ArrayList</code> do not
	 * affect the state of the <code>SongFilter</code> that this method
	 * creates and returns.
	 * </p>
	 * 
	 * @param subFilters
	 *            An <code>ArrayList</code> of <code>SongFilters</code>; to
	 *            be accepted by the newly-created <code>SongFilter</code>, a
	 *            <code>Song</code> must be accepted by at least one
	 *            <code>SongFilter</code> in this <code>ArrayList</code>.
	 * @return A new <code>SongFilter</code> which accepts a given
	 *         <code>Song</code> if and only if at least one
	 *         <code>SongFilter</code> in <code>subFilters</code> accepts
	 *         the given <code>Song</code>.
	 */
	public SongFilter createOrFilter(ArrayList<SongFilter> subFilters) {
		return new OrFilter(subFilters);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> object which accepts a given
	 * <code>Song</code> if and only if at least one of the
	 * <code>SongFilter</code>s in the specified array accepts the given
	 * <code>Song</code>.
	 * </p>
	 * 
	 * <p>
	 * Subsequent changes to the specified array do not affect the state of the
	 * <code>SongFilter</code> that this method creates and returns.
	 * </p>
	 * 
	 * @param subFilters
	 *            An array of <code>SongFilters</code>; to be accepted by the
	 *            newly-created <code>SongFilter</code>, a <code>Song</code>
	 *            must be accepted by at least one <code>SongFilter</code>s
	 *            in this array.
	 * @return A new <code>SongFilter</code> which accepts a given
	 *         <code>Song</code> if and only if at least one
	 *         <code>SongFilter</code> in <code>subFilters</code> accepts
	 *         the given <code>Song</code>.
	 */

	public SongFilter createOrFilter(SongFilter[] subFilters) {
		return new OrFilter(subFilters);
	}

	/**
	 * <p>
	 * Creates a new <code>SongFilter</code> which is the negation of the
	 * specified <code>SongFilter</code>; that is, if the specified
	 * <code>SongFilter</code> accepts a <code>Song</code>, the returned
	 * <code>SongFilter</code> rejects it, and vice-versa.
	 * </p>
	 * 
	 * @param subFilter
	 *            A <code>SongFilter</code>.
	 * @return A new <code>SongFilter</code> which accepts all
	 *         <code>Song</code>s rejected by <code>subFilter</code>, and
	 *         vice-versa.
	 */
	public SongFilter createNotFilter(SongFilter subFilter) {
		return new NotFilter(subFilter);
	}
}