package comp202.fall2007.a5.filter;

import java.util.ArrayList;

import comp202.fall2007.a5.util.Song;

abstract class MultiFilter extends SongFilter {
	private ArrayList<SongFilter> subFilters;
	private String textRepresentation;

	private void createStringRepresentation() {
		StringBuffer buffer;
		String operator;
		int numberFilters;

		operator = this.getOperator();
		numberFilters = this.subFilters.size();

		buffer = new StringBuffer();
		for (int i = 0; i < numberFilters; i++) {
			buffer.append("(" + this.subFilters.get(i) + ")");
			if (i != (numberFilters - 1)) {
				buffer.append(" " + operator + " ");
			}
		}
		this.textRepresentation = buffer.toString();
	}

	public MultiFilter(ArrayList<SongFilter> subFilters) {
		this.subFilters = new ArrayList<SongFilter>();
		this.subFilters.addAll(subFilters);
		this.createStringRepresentation();
	}

	public MultiFilter(SongFilter[] subFilters) {
		this.subFilters = new ArrayList<SongFilter>();
		for (SongFilter subFilter : subFilters) {
			this.subFilters.add(subFilter);
		}
		this.createStringRepresentation();
	}

	protected abstract boolean getInitialValue();

	protected abstract boolean updateValue(boolean left, boolean right);

	protected abstract String getOperator();

	public boolean accept(Song song) {
		SongFilter filter;
		boolean result;
		boolean initialValue;
		int i;
		int numberFilters;

		initialValue = this.getInitialValue();
		numberFilters = this.subFilters.size();

		result = initialValue;
		i = 0;
		while ((i < numberFilters) && (result == initialValue)) {
			filter = this.subFilters.get(i);
			result = this.updateValue(result, filter.accept(song));
			i = i + 1;
		}
		return result;
	}

	public String toString() {
		return this.textRepresentation;
	}
}
