package comp202.fall2007.a5.filter;

import comp202.fall2007.a5.util.Song;

/**
 * <p>
 * A <code>SongFilter</code> defines a condition that <code>Song</code>s
 * can satisfy, and provides a method to verify whether or not a
 * <code>Song</code> satisfies this condition.
 * </p>
 */
public abstract class SongFilter {
	/**
	 * Creates a new <code>SongFilter</code>.
	 */
	public SongFilter() {
	}

	/**
	 * <p>
	 * Determines whether the <code>Song</code> satisfies or passes the
	 * condition defines by this <code>SongFilter</code>. Such a
	 * <code>Song</code> is said to be accepted by the <code>SongFilter</code>;
	 * <code>Song</code>s that do not satisfy the condition defined by this
	 * <code>SongFilter</code> are said to be rejected by the latter.
	 * </p>
	 * 
	 * @param song
	 *            The <code>Song</code> to be tried with this filter.
	 * @return <code>true</code> if this <code>SongFilter</code> accepts
	 *         <code>song</code>, <code>false</code> otherwise.
	 */
	public abstract boolean accept(Song song);

	/**
	 * <p>
	 * Returns a <code>String</code> representation of this
	 * <code>SongFilter</code> in the form of a boolean expression which
	 * determines the <code>Song</code>s it accepts and those it rejects.
	 * </p>
	 * 
	 * @return A <code>String</code> representation of this
	 *         <code>SongFilter</code>.
	 */
	public abstract String toString();
}
