package comp202.fall2007.a5.loader;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;

import comp202.fall2007.a5.util.Song;
import comp202.fall2007.a5.util.SongFactory;
import comp202.fall2007.a5.util.file.DirectoryFilter;
import comp202.fall2007.a5.util.file.ExtensionFileFilter;

public class SongLoader {
	private ArrayList<SongLoaderListener> listeners;
	private SongFactory songFactory;
	private volatile boolean stop;

	private static final FileFilter DIRECTORY_FILTER = new DirectoryFilter();
	private static final FileFilter MP3_FILE_FILTER = new ExtensionFileFilter(
			".MP3");

	public SongLoader() {
		this.listeners = new ArrayList<SongLoaderListener>();
		this.songFactory = new SongFactory();
		this.stop = false;
	}

	public void addListener(SongLoaderListener listener) {
		this.listeners.add(listener);
	}

	public void removeListener(SongLoaderListener listener) {
		this.listeners.remove(listener);
	}

	public void clearListener() {
		this.listeners.clear();
	}

	private void loadAll(File directory) {
		File[] directoryList;
		File[] fileList;
		Song song;
		int i;

		directoryList = directory.listFiles(DIRECTORY_FILTER);
		i = 0;
		while ((i < directoryList.length) && (!this.stop)) {
			this.loadAll(directoryList[i++]);
		}

		fileList = directory.listFiles(MP3_FILE_FILTER);
		i = 0;
		while ((i < fileList.length) && (!this.stop)) {
			try {
				song = this.songFactory.createSong(fileList[i++]
						.getAbsolutePath());

				for (SongLoaderListener listener : this.listeners) {
					listener.loadedSong(song);
				}
			} catch (IOException exception) {
			}
		}
	}

	public void load(File rootDirectory) {
		if (!rootDirectory.isDirectory()) {
			throw new IllegalArgumentException(rootDirectory
					+ " is not a directory");
		}

		for (SongLoaderListener listener : this.listeners) {
			listener.startedLoading();
		}

		this.loadAll(rootDirectory);

		if (!this.stop) {
			for (SongLoaderListener listener : this.listeners) {
				listener.finishedLoading();
			}
		} else {
			for (SongLoaderListener listener : this.listeners) {
				listener.loadingInterrupted();
			}
		}
	}
}