package comp202.fall2007.a5.util;

import java.io.File;

/**
 * <p>
 * A <code>Song</code> object records the attributes of a digital audio track.
 * These attributes includes the title of the audio track, the artist who
 * produced it, the album on which it appears, the disc number, the track
 * number, the playing length of the the track, and the path of the file
 * on disk which contains this track.
 * </p>
 * 
 * <p>
 * <code>Song</code> objects are <i>immutable</i>; that is, the value of
 * their attributes cannot be changed after they have been created.
 * </p>
 */
public class Song {
	/**
	 * Constant indicating that the track attribute is undefined.
	 */
	public static final int EMPTY_TRACK = -1;

	private String title;
	private String artist;
	private String album;
	private int discNumber;
	private int trackNumber;
	private int length;
	private File filePath;
	private String textRepresentation;

	/**
	 * Creates a new <code>Song</code> object with the specified attributes.
	 * 
	 * @param filePath
	 *            The path of the file on disk containing the audio track
	 *            represented by the newly-created <code>Song</code> object.
	 * @param artist
	 *            The artist who produced the newly-created <code>Song</code>.
	 * @param album
	 *            The album on which the newly-created <code>Song</code> appears.
	 * @param title
	 *            The title of the newly-created <code>Song</code>.
	 * @param discNumber
	 *            The disc number of the newly-created <code>Song</code>.
	 * @param trackNumber
	 *            The track number of the newly-created <code>Song</code>.
	 * @param length
	 *            The length in seconds of the audio track represented by the
	 *            newly-created <code>Song</code>.
	 */
	public Song(File filePath, String artist, String album, String title,
			int discNumber, int trackNumber, int length) {
		this.filePath = filePath;
		this.artist = artist;
		this.album = album;
		this.title = title;
		this.trackNumber = trackNumber;
		this.discNumber = discNumber;
		this.length = length;

		this.textRepresentation = this.title + " (" + /*
														 * this.artist + " - " +
														 * this.album + " - "
														 */+this.discNumber + "-" + this.trackNumber + ")";
	}

	/**
	 * Creates a new <code>Song</code> object with the specified attributes,
	 * and a disc number attribute equal to <code>1</code>.
	 * 
	 * @param filePath
	 *            The path of the file on disk containing the audio track
	 *            represented by the newly-created <code>Song</code> object.
	 * @param artist
	 *            The name of the artist who produced the newly-created
	 *            <code>Song</code>.
	 * @param album
	 *            The title of the album on which the newly-created
	 *            <code>Song</code> appears.
	 * @param title
	 *            The title of the newly-created <code>Song</code>.
	 * @param trackNumber
	 *            The track number of the newly-created <code>Song</code>.
	 * @param length
	 *            The length in seconds of the audio track represented by the
	 *            newly-created <code>Song</code>.
	 */
	public Song(File filePath, String artist, String album, String title,
			int trackNumber, int length) {
		this(filePath, artist, album, title, 1, trackNumber, length);
	}

	/**
	 * Returns the path of the file on disk containing the audio track
	 * represented by this <code>Song</code> object.
	 * 
	 * @return The path of the file on disk containing the audio track
	 *         represented by this <code>Song</code> object.
	 */
	public File getFilePath() {
		return this.filePath;
	}

	/**
	 * Returns the name of the artist who produced this <code>Song</code>.
	 * 
	 * @return The name of the artist who produced this <code>Song</code>.
	 */
	public String getArtist() {
		return this.artist;
	}

	/**
	 * Returns the title of the album on which this <code>Song</code> appears.
	 * 
	 * @return The title of the album on which this <code>Song</code> appears.
	 */
	public String getAlbum() {
		return this.album;
	}

	/**
	 * Returns the title of this <code>Song</code>.
	 * 
	 * @return The title of this <code>Song</code>.
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Returns the track number of this <code>Song</code>.
	 * 
	 * @return The track number of this <code>Song</code>.
	 */
	public int getTrackNumber() {
		return this.trackNumber;
	}

	/**
	 * Returns the disc number of this <code>Song</code>.
	 * 
	 * @return The disc number of this <code>Song</code>.
	 */
	public int getDiscNumber() {
		return this.discNumber;
	}

	/**
	 * Returns the length of this <code>Song</code> in seconds.
	 * 
	 * @return The length of this <code>Song</code> in seconds.
	 */
	public int getLength() {
		return this.length;
	}

	/**
	 * Returns a <code>String</code> representation of this <code>Song</code>.
	 * 
	 * @return A <code>String</code> representation of this <code>Song</code>.
	 */
	public String toString() {
		return this.textRepresentation;
	}
}